@@ -7,7 +7,7 @@ Attean::API - Utility package for loading all Attean role packages.
 
 =head1 VERSION
 
-This document describes Attean::API version 0.009
+This document describes Attean::API version 0.010
 
 =head1 SYNOPSIS
 
@@ -21,15 +21,15 @@ in the Attean::API namespace.
 
 =cut
 
-package Attean::API::ResultOrTerm 0.009 {
+package Attean::API::ResultOrTerm 0.010 {
 	use Moo::Role;
 }
 
-package Attean::API::BlankOrIRI 0.009 {
+package Attean::API::BlankOrIRI 0.010 {
 	use Moo::Role;
 }
 
-package Attean::API::TermOrVariable 0.009 {
+package Attean::API::TermOrVariable 0.010 {
 	use Moo::Role;
 	use Sub::Install;
 	use Sub::Util qw(set_subname);
@@ -59,14 +59,20 @@ package Attean::API::TermOrVariable 0.009 {
 	}
 }
 
-package Attean::Mapper 0.009 {
+package Attean::Mapper 0.010 {
 	use Moo::Role;
 	requires 'map'; # my $that = $object->map($this)
 }
 
-package Attean::API::Variable 0.009 {
+package Attean::API::Variable 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo::Role;
+	use namespace::clean;
+
 	with 'Attean::API::TermOrVariable';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub as_sparql {
 		my $self	= shift;
 		return '?' . $self->value;
@@ -76,9 +82,16 @@ package Attean::API::Variable 0.009 {
 		my $self	= shift;
 		return '?' . $self->value;
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $t	= AtteanX::SPARQL::Token->fast_constructor( VAR, -1, -1, -1, -1, [$self->value] );
+		return Attean::ListIterator->new( values => [$t], item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 }
 
-package Attean::API::CanonicalizingBindingSet 0.009 {
+package Attean::API::CanonicalizingBindingSet 0.010 {
 	use Moo::Role;
 	use Attean::RDF;
 	use namespace::clean;
@@ -169,7 +182,7 @@ package Attean::API::CanonicalizingBindingSet 0.009 {
 	}
 }
 
-package Attean::API 0.009 {
+package Attean::API 0.010 {
 	use Attean::API::Term;
 	use Attean::API::Store;
 	use Attean::API::Model;
@@ -179,6 +192,7 @@ package Attean::API 0.009 {
 	use Attean::API::Query;
 	use Attean::API::Expression;
 	use Attean::API::Plan;
+	use Attean::API::QueryPlanner;
 	
 	use Attean::Variable;
 	use Attean::Blank;
