@@ -7,7 +7,7 @@ Attean::API::Binding - Name to term bindings
 
 =head1 VERSION
 
-This document describes Attean::API::Binding version 0.009
+This document describes Attean::API::Binding version 0.010
 
 =head1 DESCRIPTION
 
@@ -71,7 +71,7 @@ otherwise.
 
 use Type::Tiny::Role;
 
-package Attean::API::Binding 0.009 {
+package Attean::API::Binding 0.010 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use List::MoreUtils qw(zip);
@@ -148,12 +148,24 @@ package Attean::API::Binding 0.009 {
 	}
 }
 
-package Attean::API::TripleOrQuadPattern 0.009 {
+package Attean::API::TripleOrQuadPattern 0.010 {
 	use Scalar::Util qw(blessed);
 	use Attean::RDF;
+	use Attean::API::Query;
 	use Moo::Role;
 	use namespace::clean;
 
+	with 'Attean::API::SPARQLSerializable';
+	
+	sub sparql_tokens {
+		my $self	= shift;
+		my @tokens;
+		foreach my $t ($self->values) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 	around BUILDARGS => sub {
 		my $orig 	= shift;
 		my $class	= shift;
@@ -215,12 +227,12 @@ package Attean::API::TripleOrQuadPattern 0.009 {
 	}
 }
 
-package Attean::API::TripleOrQuad 0.009 {
+package Attean::API::TripleOrQuad 0.010 {
 	use Moo::Role;
 	with 'Attean::API::TripleOrQuadPattern';
 }
 
-package Attean::API::TriplePattern 0.009 {
+package Attean::API::TriplePattern 0.010 {
 	use Moo::Role;
 	use List::MoreUtils qw(zip);
 	use Scalar::Util qw(blessed);
@@ -268,7 +280,7 @@ package Attean::API::TriplePattern 0.009 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Triple 0.009 {
+package Attean::API::Triple 0.010 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -303,7 +315,7 @@ package Attean::API::Triple 0.009 {
 	with 'Attean::API::TriplePattern', 'Attean::API::TripleOrQuad', 'Attean::API::Binding';
 }
 
-package Attean::API::QuadPattern 0.009 {
+package Attean::API::QuadPattern 0.010 {
 	use Scalar::Util qw(blessed);
 	use Moo::Role;
 	
@@ -336,7 +348,7 @@ package Attean::API::QuadPattern 0.009 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Quad 0.009 {
+package Attean::API::Quad 0.010 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -363,7 +375,7 @@ package Attean::API::Quad 0.009 {
 }
 
 
-package Attean::API::Result 0.009 {
+package Attean::API::Result 0.010 {
 	use Moo::Role;
 	use Scalar::Util qw(refaddr);
 	use Types::Standard qw(HashRef);
