@@ -7,7 +7,7 @@ Attean::API::Plan - Query plan
 
 =head1 VERSION
 
-This document describes Attean::API::Plan version 0.009
+This document describes Attean::API::Plan version 0.010
 
 =head1 DESCRIPTION
 
@@ -23,13 +23,13 @@ The following methods are required by the L<Attean::API::Plan> role:
 
 use Type::Tiny::Role;
 
-package Attean::API::Plan 0.009 {
+package Attean::API::Plan 0.010 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
-	use Types::Standard qw(ArrayRef CodeRef Str Object InstanceOf Bool Num);
+	use Types::Standard qw(ArrayRef CodeRef Str Object InstanceOf Bool Num Int);
 	use namespace::clean;
 	
-	has 'cost' => (is => 'rw', isa => Num, predicate => 'has_cost');
+	has 'cost' => (is => 'rw', isa => Int, predicate => 'has_cost');
 	has 'distinct' => (is => 'rw', isa => Bool, required => 1, default => 0);
 	has 'item_type' => (is => 'ro', isa => Str, required => 1, default => 'Attean::API::Result');
 	has 'in_scope_variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
@@ -61,7 +61,7 @@ package Attean::API::Plan 0.009 {
 				}
 				push(@flags, "order: " . join('; ', @orders));
 			}
-			$string	.= "-$indent " .  $a->plan_as_string;
+			$string	.= "-$indent " .  $a->plan_as_string($level);
 			if (scalar(@flags)) {
 				$string .= ' (' . join(' ', @flags) . ")";
 			}
@@ -83,7 +83,7 @@ package Attean::API::Plan 0.009 {
 	}
 }
 
-package Attean::API::BindingSubstitutionPlan 0.009 {
+package Attean::API::BindingSubstitutionPlan 0.010 {
 	use Moo::Role;
 	with 'Attean::API::Plan';
 	requires 'substitute_impl'; # $code = $plan->impl($model, $binding);
@@ -96,19 +96,12 @@ package Attean::API::BindingSubstitutionPlan 0.009 {
 	}
 }
 
-package Attean::API::Plan::Join 0.009 {
+package Attean::API::UnionScopeVariablesPlan 0.010 {
 	use Moo::Role;
-	use Types::Standard qw(CodeRef);
 	use namespace::clean;
-	use Types::Standard qw(ArrayRef Str ConsumerOf Bool);
-	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
-	has 'join_variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
-	has 'anti' => (is => 'ro', isa => Bool, default => 0);	# is this an anti-join
-	has 'left' => (is => 'ro', isa => Bool, default => 0);	# is this a left, outer-join
-	
-	# if this is a left, outer-join, this is the filter expression that acts as part of the join operation (see the SPARQL semantics for LeftJoin for more details)
-	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 0, default => sub { Attean::ValueExpression->new( value => Attean::Literal->true ) });
 
+	with 'Attean::API::Plan';
+	
 	around 'BUILDARGS' => sub {
 		my $orig		= shift;
 		my $class		= shift;
@@ -121,37 +114,24 @@ package Attean::API::Plan::Join 0.009 {
 		$args{in_scope_variables}	= [@vars];
 
 		return $orig->( $class, %args );
-	}
-	
+	};
 }
 
-package Attean::API::Planner 0.009 {
+package Attean::API::Plan::Join 0.010 {
 	use Moo::Role;
 	use Types::Standard qw(CodeRef);
+	use Types::Standard qw(ArrayRef Str ConsumerOf Bool);
 	use namespace::clean;
-	
-	requires 'plan_for_algebra'; # plan_for_algebra($algebra, $model, \@default_graphs)
-}
 
-package Attean::API::CostPlanner 0.009 {
-	use Moo::Role;
-	use Types::Standard qw(CodeRef);
-	use namespace::clean;
-	with 'Attean::API::Planner';
+	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
 	
-	requires 'plans_for_algebra'; # plans_for_algebra($algebra, $model, \@active_graphs, \@default_graphs)
-	requires 'cost_for_plan'; # cost_for_plan($plan, $model)
+	has 'join_variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
+	has 'anti' => (is => 'ro', isa => Bool, default => 0);	# is this an anti-join
+	has 'left' => (is => 'ro', isa => Bool, default => 0);	# is this a left, outer-join
 	
-	sub plan_for_algebra {
-		my $self			= shift;
-		my $algebra			= shift;
-		my $model			= shift;
-		my $default_graphs	= shift;
-		my $active_graphs	= $default_graphs;
-		my @plans			= sort { $self->cost_for_plan($a, $model) <=> $self->cost_for_plan($b, $model) } $self->plans_for_algebra($algebra, $model, $active_graphs, $default_graphs);
-		my $plan			= shift(@plans);
-		return $plan;
-	}
+	# if this is a left, outer-join, this is the filter expression that acts as part of the join operation (see the SPARQL semantics for LeftJoin for more details)
+	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 0, default => sub { Attean::ValueExpression->new( value => Attean::Literal->true ) });
 }
 
 1;
