@@ -3,20 +3,20 @@ use warnings;
 
 =head1 NAME
 
-Attean::API::IDPJoinPlanner - Iterative dynamic programming query planning role
+Attean::API::IDPJoinPlanning - Iterative dynamic programming query planning role
 
 =head1 VERSION
 
-This document describes Attean::API::IDPJoinPlanner version 0.034
+This document describes Attean::API::IDPJoinPlanning version 0.010
 
 =head1 SYNOPSIS
 
   extends 'Attean::QueryPlanner';
-  with 'Attean::API::IDPJoinPlanner';
+  with 'Attean::API::IDPJoinPlanning';
 
 =head1 DESCRIPTION
 
-The Attean::API::IDPJoinPlanner role provides a query planner the
+The Attean::API::IDPJoinPlanning role provides a query planner the
 C<< joins_for_plan_alternatives >> method, as well as the cost estimation
 methods that consume the L<Attean::API::CostPlanner> role.
 
@@ -32,38 +32,23 @@ methods that consume the L<Attean::API::CostPlanner> role.
 
 =cut
 
-package Attean::API::QueryPlanner 0.034 {
-	use Types::Standard qw(CodeRef Object);
-
+package Attean::API::QueryPlanner 0.010 {
 	use Moo::Role;
+	use Types::Standard qw(CodeRef);
+	use namespace::clean;
 	
 	requires 'plan_for_algebra'; # plan_for_algebra($algebra, $model, \@default_graphs)
-	has 'request_signer' => (is => 'rw', isa => Object);
 }
 
-package Attean::API::CostPlanner 0.034 {
-	use Scalar::Util qw(refaddr);
-	use Types::Standard qw(CodeRef);
-
+package Attean::API::CostPlanner 0.010 {
 	use Moo::Role;
+	use Types::Standard qw(CodeRef);
 	use namespace::clean;
 	with 'Attean::API::QueryPlanner';
 	
 	requires 'plans_for_algebra'; # plans_for_algebra($algebra, $model, \@active_graphs, \@default_graphs)
 	requires 'cost_for_plan'; # cost_for_plan($plan, $model)
 	
-	before 'cost_for_plan' => sub {
-		my $self	= shift;
-		my $plan	= shift;
-		my $model	= shift;
-		
-		if (refaddr($self) == refaddr($model)) {
-			Carp::confess "Model and planner objects cannot be the same in call to cost_for_plan";
-		} elsif ($self->does('Attean::API::Model') and $model->does('Attean::API::Model')) {
-			Carp::confess "Model and planner objects cannot both consume Attean::API::Model in call to cost_for_plan";
-		}
-	};
-	
 	sub plan_for_algebra {
 		my $self			= shift;
 		my $algebra			= shift;
@@ -76,15 +61,16 @@ package Attean::API::CostPlanner 0.034 {
 	}
 }
 
-package Attean::API::JoinPlanner 0.034 {
+package Attean::API::JoinPlanner 0.010 {
 	use Moo::Role;
+	use namespace::clean;
 	requires 'joins_for_plan_alternatives';
 }
 
-package Attean::API::NaiveJoinPlanner 0.034 {
-	use Math::Cartesian::Product;
-
+package Attean::API::NaiveJoinPlanner 0.010 {
 	use Moo::Role;
+	use Math::Cartesian::Product;
+	use namespace::clean;
 
 	with 'Attean::API::JoinPlanner';
 	with 'Attean::API::QueryPlanner';
@@ -109,14 +95,12 @@ package Attean::API::NaiveJoinPlanner 0.034 {
 	}
 }
 
-package Attean::API::SimpleCostPlanner 0.034 {
+package Attean::API::SimpleCostPlanner 0.010 {
+	use Moo::Role;
+	use namespace::clean;
 	use Types::Standard qw(Int);
 	use Scalar::Util qw(blessed);
-
-	use Moo::Role;
-
 	with 'Attean::API::CostPlanner';
-	with 'MooX::Log::Any';
 
 	has 'keep' => (is => 'ro', isa => Int, default => 5);
 	
@@ -143,15 +127,14 @@ package Attean::API::SimpleCostPlanner 0.034 {
 		my $self	= shift;
 		my $plan	= shift;
 		my $model	= shift;
-		Carp::confess "No model given" unless (blessed($model) and $model->does('Attean::API::Model'));
+		Carp::confess "No model given" unless ref($model);
 		
 		if ($plan->has_cost) {
 			return $plan->cost;
 		} else {
 			if ($model->does('Attean::API::CostPlanner')) {
-				if (defined(my $cost = $model->cost_for_plan($plan, $self))) {
+				if (defined(my $cost = $model->cost_for_plan($plan, $model))) {
 					$plan->cost($cost);
-					$self->log->info('Model \''.ref($model).'\' did cost planning for \''.ref($plan).'\' and got cost '.$cost);
 					return $cost;
 				}
 			}
@@ -174,18 +157,16 @@ package Attean::API::SimpleCostPlanner 0.034 {
 				} else {
 					$cost	= $lcost * $rcost;
 				}
-				
-				# a cartesian nested loop join is bad, but the algorithm already
-				# has to check for all possible joins, so it's not as bad as
-				# a cartesian hash join (below)
-				$cost	*= 10 unless ($plan->children_are_variable_connected);
 			} elsif ($plan->isa('Attean::Plan::HashJoin')) {
-				my $joined		= $plan->children_are_variable_connected;
 				my $lcost		= $self->cost_for_plan($children[0], $model);
 				my $rcost		= $self->cost_for_plan($children[1], $model);
-				$cost	= ($lcost + $rcost);
-				$cost += ($lcost < $rcost); # To let the plan with cheaper rhs win
-				$cost	*= 100 unless ($plan->children_are_variable_connected);
+				if ($lcost == 0) {
+					$cost	= $rcost;
+				} elsif ($rcost == 0) {
+					$cost	= $lcost;
+				} else {
+					$cost	= ($lcost + $rcost);
+				}
 			} elsif ($plan->isa('Attean::Plan::Service')) {
 				my $scost	= 10;
 				foreach my $c (@{ $plan->children }) {
@@ -204,30 +185,25 @@ package Attean::API::SimpleCostPlanner 0.034 {
 			}
 			
 			$plan->cost($cost);
-			if ($self->log->is_trace) {
-				$self->log->trace("Cost $cost estimated for\n".$plan->as_string);
-			} else {
-				$self->log->debug('Estimated cost for \''.ref($plan).'\' is '.$cost);
-			}
 			return $cost;
 		}
 	}
 }
 
-package Attean::API::IDPJoinPlanner 0.034 {
+package Attean::API::IDPJoinPlanner 0.010 {
+	use Moo::Role;
 	use Encode qw(encode);
 	use Attean::RDF;
 	use LWP::UserAgent;
 	use Scalar::Util qw(blessed reftype);
 	use List::Util qw(reduce);
-	use List::Util qw(all any);
+	use List::MoreUtils qw(all any);
 	use Types::Standard qw(Int ConsumerOf InstanceOf);
 	use URI::Escape;
 	use Algorithm::Combinatorics qw(subsets);
 	use List::Util qw(min);
 	use Math::Cartesian::Product;
-
-	use Moo::Role;
+	use namespace::clean;
 
 	with 'Attean::API::JoinPlanner';
 	with 'Attean::API::SimpleCostPlanner';
@@ -337,12 +313,6 @@ package Attean::API::IDPJoinPlanner 0.034 {
 		my @plans		= @{ shift || [] };
 		no  sort 'stable';
 		my @sorted	= map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [$self->cost_for_plan($_, $model), $_] } @plans;
-		if ($self->log->is_trace) {
-			$self->log->trace('============= Plan iteration separator ==============');
-			foreach my $plan (@sorted){
-				$self->log->trace("Cost: " . $self->cost_for_plan($plan, $model) . " for plan:\n". $plan->as_string);
-			}
-		}
 		return splice(@sorted, 0, 5);
 	}
 	
@@ -353,15 +323,14 @@ package Attean::API::IDPJoinPlanner 0.034 {
 		my $self	= shift;
 		my $plan	= shift;
 		my $model	= shift;
-		Carp::confess "No model given" unless (blessed($model) and $model->does('Attean::API::Model'));
+		Carp::confess "No model given" unless ref($model);
 		
 		if ($plan->has_cost) {
 			return $plan->cost;
 		} else {
 			if ($model->does('Attean::API::CostPlanner')) {
-				if (defined(my $cost = $model->cost_for_plan($plan, $self))) {
+				if (defined(my $cost = $model->cost_for_plan($plan, $model))) {
 					$plan->cost($cost);
-					$self->log->info('Model \''.ref($model).'\' did cost planning for \''.ref($plan).'\' and got cost '.$cost);
 					return $cost;
 				}
 			}
@@ -398,7 +367,6 @@ package Attean::API::IDPJoinPlanner 0.034 {
 					my $mult = $self->_penalize_joins($plan);
 # 					warn "$mult * ($lcost + $rcost)";
 					$cost	= $mult * ($lcost + $rcost);
-					$cost += ($lcost < $rcost); # To let the plan with cheaper rhs win
 				}
 			} elsif ($plan->isa('Attean::Plan::Service')) {
 				my $scost	= 10;
@@ -555,7 +523,7 @@ influenced by L<The ICS-FORTH Heuristics-based SPARQL Planner
 
 =head1 SEE ALSO
 
-
+L<http://www.perlrdf.org/>
 
 =head1 AUTHOR
 
@@ -563,7 +531,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Gregory Todd Williams.
+Copyright (c) 2014 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
