@@ -7,7 +7,7 @@ Attean::Algebra - Representation of SPARQL algebra operators
 
 =head1 VERSION
 
-This document describes Attean::Algebra version 0.009
+This document describes Attean::Algebra version 0.010
 
 =head1 SYNOPSIS
 
@@ -25,14 +25,69 @@ in the Attean::Algebra namespace:
 
 use Attean::API::Query;
 
+package Attean::Algebra::Sequence 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use Moo;
+	use namespace::clean;
+
+	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::QueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
+	sub algebra_as_string { return 'Sequence' }
+	sub as_sparql {
+		my $self	= shift;
+		my %args	= @_;
+		my $level	= $args{level} // 0;
+		my $sp		= $args{indent} // '    ';
+		my $indent	= $sp x $level;
+		
+		return join(";\n", map { $_->as_sparql( %args ) } @{ $self->children });
+	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $semi	= AtteanX::SPARQL::Token->fast_constructor( SEMICOLON, -1, -1, -1, -1, [';'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $semi);
+		}
+		pop(@tokens); # remove last SEMICOLON token
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+}
+
 =item * L<Attean::Algebra::Join>
 
 =cut
 
-package Attean::Algebra::Join 0.009 {
+package Attean::Algebra::Join 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
-	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::BinaryQueryTree';
+	use namespace::clean;
+
+	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::QueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub algebra_as_string { return 'Join' }
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $l	= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r	= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+
+		my @tokens;
+		push(@tokens, $l);
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 	sub as_sparql {
 		my $self	= shift;
 		my %args	= @_;
@@ -48,10 +103,16 @@ package Attean::Algebra::Join 0.009 {
 
 =cut
 
-package Attean::Algebra::LeftJoin 0.009 {
+package Attean::Algebra::LeftJoin 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::BinaryQueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
 	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 1, default => sub { Attean::ValueExpression->new( value => Attean::Literal->true ) });
 	sub algebra_as_string {
 		my $self	= shift;
@@ -79,13 +140,30 @@ package Attean::Algebra::LeftJoin 0.009 {
 		$s	.= "${indent}}\n";
 		return $s;
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $opt	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['OPTIONAL'] );
+		my $l	= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r	= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+		my ($lhs, $rhs)	= @{ $self->children };
+		
+		my @tokens;
+		push(@tokens, $l);
+		push(@tokens, $lhs->sparql_tokens->elements);
+		push(@tokens, $r, $opt, $l);
+		push(@tokens, $rhs->sparql_tokens->elements);
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 }
 
 =item * L<Attean::Algebra::Filter>
 
 =cut
 
-package Attean::Algebra::Filter 0.009 {
+package Attean::Algebra::Filter 0.010 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
@@ -113,9 +191,16 @@ package Attean::Algebra::Filter 0.009 {
 
 =cut
 
-package Attean::Algebra::Union 0.009 {
+package Attean::Algebra::Union 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
+	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::BinaryQueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub algebra_as_string { return 'Union' }
 	sub as_sparql {
 		my $self	= shift;
@@ -131,22 +216,47 @@ package Attean::Algebra::Union 0.009 {
 			. $rhs->as_sparql( %args, level => $level+1 )
 			. "${indent}}\n";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $union	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['UNION'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+		my ($lhs, $rhs)	= @{ $self->children };
+		
+		my @tokens;
+		push(@tokens, $l);
+		push(@tokens, $lhs->sparql_tokens->elements);
+		push(@tokens, $r, $union, $l);
+		push(@tokens, $rhs->sparql_tokens->elements);
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Graph>
 
 =cut
 
-package Attean::Algebra::Graph 0.009 {
+package Attean::Algebra::Graph 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
+	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
+	has 'graph' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
+
 	sub in_scope_variables {
 		my $self	= shift;
 		my $graph	= $self->graph;
 		my ($child)	= @{ $self->children };
 		my @vars	= $child->in_scope_variables;
 		if ($graph->does('Attean::API::Variable')) {
-			return Set::Scalar->new(@vars, $graph->value)->members;
+			return Set::Scalar->new(@vars, $graph->value)->elements;
 		} else {
 			return @vars;
 		}
@@ -155,8 +265,6 @@ package Attean::Algebra::Graph 0.009 {
 		my $self	= shift;
 		return sprintf('Graph %s', $self->graph->as_string);
 	}
-	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	has 'graph' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
 	sub tree_attributes { return qw(graph) };
 	sub as_sparql {
 		my $self	= shift;
@@ -171,20 +279,36 @@ package Attean::Algebra::Graph 0.009 {
 			. $child->as_sparql( %args, level => $level+1 )
 			. "${indent}}\n";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $graph	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['GRAPH'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+		my ($child)	= @{ $self->children };
+		
+		my @tokens;
+		push(@tokens, $graph);
+		push(@tokens, $self->graph->sparql_tokens->elements);
+		push(@tokens, $l);
+		push(@tokens, $child->sparql_tokens->elements);
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Extend>
 
 =cut
 
-package Attean::Algebra::Extend 0.009 {
+package Attean::Algebra::Extend 0.010 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
 	sub in_scope_variables {
 		my $self	= shift;
 		my ($child)	= @{ $self->children };
 		my @vars	= $child->in_scope_variables;
-		return Set::Scalar->new(@vars, $self->variable->value)->members;
+		return Set::Scalar->new(@vars, $self->variable->value)->elements;
 	}
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
 	has 'variable' => (is => 'ro', isa => ConsumerOf['Attean::API::Variable'], required => 1);
@@ -231,14 +355,22 @@ package Attean::Algebra::Extend 0.009 {
 
 =cut
 
-package Attean::Algebra::Minus 0.009 {
+package Attean::Algebra::Minus 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
+	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
+	with 'Attean::API::Algebra', 'Attean::API::BinaryQueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub in_scope_variables {
 		my $self	= shift;
 		my ($child)	= @{ $self->children };
 		return $child->in_scope_variables;
 	}
-	with 'Attean::API::Algebra', 'Attean::API::BinaryQueryTree';
+
 	sub algebra_as_string { return 'Minus' }
 	sub as_sparql {
 		my $self	= shift;
@@ -254,15 +386,36 @@ package Attean::Algebra::Minus 0.009 {
 			. $rhs->as_sparql( %args, level => $level+1 )
 			. "${indent}}\n";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $minus	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['MINUS'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+		my ($lhs, $rhs)	= @{ $self->children };
+		
+		my @tokens;
+		push(@tokens, $l);
+		push(@tokens, $lhs->sparql_tokens->elements);
+		push(@tokens, $r, $minus, $l);
+		push(@tokens, $rhs->sparql_tokens->elements);
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 }
 
 =item * L<Attean::Algebra::Distinct>
 
 =cut
 
-package Attean::Algebra::Distinct 0.009 {
+package Attean::Algebra::Distinct 0.010 {
 	use Moo;
+	use namespace::clean;
+
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
 	sub algebra_as_string { return 'Distinct' }
 	sub as_sparql {
 		my $self	= shift;
@@ -286,9 +439,13 @@ package Attean::Algebra::Distinct 0.009 {
 
 =cut
 
-package Attean::Algebra::Reduced 0.009 {
+package Attean::Algebra::Reduced 0.010 {
 	use Moo;
+	use namespace::clean;
+
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
 	sub algebra_as_string { return 'Reduced' }
 	sub as_sparql {
 		my $self	= shift;
@@ -312,10 +469,14 @@ package Attean::Algebra::Reduced 0.009 {
 
 =cut
 
-package Attean::Algebra::Slice 0.009 {
+package Attean::Algebra::Slice 0.010 {
 	use Moo;
 	use Types::Standard qw(Int);
+	use namespace::clean;
+
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
 	has 'limit' => (is => 'ro', isa => Int, default => -1);
 	has 'offset' => (is => 'ro', isa => Int, default => 0);
 	sub algebra_as_string {
@@ -325,6 +486,7 @@ package Attean::Algebra::Slice 0.009 {
 		push(@str, "Offset=" . $self->offset) if ($self->offset > 0);
 		return join(' ', @str);
 	}
+
 	sub as_sparql {
 		my $self	= shift;
 		my %args	= @_;
@@ -353,18 +515,23 @@ package Attean::Algebra::Slice 0.009 {
 
 =cut
 
-package Attean::Algebra::Project 0.009 {
+package Attean::Algebra::Project 0.010 {
+	use Types::Standard qw(ArrayRef ConsumerOf);
 	use Moo;
+	use namespace::clean;
+
+	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
+	has 'variables' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']], required => 1);
+
 	sub in_scope_variables {
 		my $self	= shift;
 		my ($child)	= @{ $self->children };
 		my $set		= Set::Scalar->new( $child->in_scope_variables );
 		my $proj	= Set::Scalar->new( map { $_->value } @{ $self->variables } );
-		return $set->intersection($proj);
+		return $set->intersection($proj)->elements;
 	}
-	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	use Types::Standard qw(ArrayRef ConsumerOf);
-	has 'variables' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']], required => 1);
 	sub algebra_as_string {
 		my $self	= shift;
 		return sprintf('Project { %s }', join(' ', map { '?' . $_->value } @{ $self->variables }));
@@ -412,11 +579,18 @@ package Attean::Algebra::Project 0.009 {
 
 =cut
 
-package Attean::Algebra::Comparator 0.009 {
+package Attean::Algebra::Comparator 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(Bool ConsumerOf);
+	use namespace::clean;
+
+	with 'Attean::API::SPARQLSerializable';
+
 	has 'ascending' => (is => 'ro', isa => Bool, default => 1);
 	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 1);
+
 	sub tree_attributes { return qw(expression) };
 	sub as_string {
 		my $self	= shift;
@@ -434,17 +608,42 @@ package Attean::Algebra::Comparator 0.009 {
 			return 'DESC(' . $self->expression->as_sparql . ')';
 		}
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $asc		= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['ASC'] );
+		my $desc	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['DESC'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+		
+		my @tokens;
+		if ($self->ascending) {
+			push(@tokens, $self->expression->sparql_tokens->elements);
+		} else {
+			push(@tokens, $desc, $l);
+			push(@tokens, $self->expression->sparql_tokens->elements);
+			push(@tokens, $r);
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::OrderBy>
 
 =cut
 
-package Attean::Algebra::OrderBy 0.009 {
+package Attean::Algebra::OrderBy 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef InstanceOf);
+	use namespace::clean;
+	
 	with 'Attean::API::UnionScopeVariables', 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+	
 	has 'comparators' => (is => 'ro', isa => ArrayRef[InstanceOf['Attean::Algebra::Comparator']], required => 1);
+	
 	sub tree_attributes { return qw(comparators) };
 	sub algebra_as_string {
 		my $self	= shift;
@@ -469,13 +668,20 @@ package Attean::Algebra::OrderBy 0.009 {
 
 =cut
 
-package Attean::Algebra::BGP 0.009 {
+package Attean::Algebra::BGP 0.010 {
 	use Moo;
 	use Attean::RDF;
 	use Set::Scalar;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
 
+	with 'Attean::API::Algebra', 'Attean::API::NullaryQueryTree', 'Attean::API::CanonicalizingBindingSet';
+	with 'Attean::API::SPARQLSerializable';
+	
+	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']], default => sub { [] });
+	
 	sub in_scope_variables {
 		my $self	= shift;
 		my $set		= Set::Scalar->new();
@@ -483,7 +689,18 @@ package Attean::Algebra::BGP 0.009 {
 			my @vars	= map { $_->value } grep { $_->does('Attean::API::Variable') } $t->values;
 			$set->insert(@vars);
 		}
-		return $set->members;
+		return $set->elements;
+	}
+	
+	sub sparql_tokens {
+		my $self	= shift;
+		my @tokens;
+		my $dot	= AtteanX::SPARQL::Token->fast_constructor( DOT, -1, -1, -1, -1, ['.'] );
+		foreach my $t (@{ $self->triples }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $dot);
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
 	}
 	
 	sub as_sparql {
@@ -503,15 +720,11 @@ package Attean::Algebra::BGP 0.009 {
 		return 'BGP { ' . join(', ', map { $_->as_string } @{ $self->triples }) . ' }';
 	}
 	
-	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']], default => sub { [] });
-	
 	sub elements {
 		my $self	= shift;
 		return @{ $self->triples };
 	}
 	
-	with 'Attean::API::Algebra', 'Attean::API::NullaryQueryTree', 'Attean::API::CanonicalizingBindingSet';
-
 	sub canonicalize {
 		my $self	= shift;
 		my ($algebra, $mapping)	= $self->canonical_bgp_with_mapping();
@@ -538,17 +751,24 @@ package Attean::Algebra::BGP 0.009 {
 
 =cut
 
-package Attean::Algebra::Service 0.009 {
+package Attean::Algebra::Service 0.010 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool);
+	use namespace::clean;
+
+	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree', 'Attean::API::UnionScopeVariables';
+	with 'Attean::API::SPARQLSerializable';
+
+	has 'endpoint' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
+	has 'silent' => (is => 'ro', isa => Bool, default => 0);
+	
 	sub algebra_as_string {
 		my $self	= shift;
 		return sprintf('Service %s', $self->endpoint->as_string);
 	}
-	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree', 'Attean::API::UnionScopeVariables';
-	has 'endpoint' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
-	has 'silent' => (is => 'ro', isa => Bool, default => 0);
-	
+
 	sub tree_attributes { return qw(endpoint) };
 	sub as_sparql {
 		my $self	= shift;
@@ -563,24 +783,48 @@ package Attean::Algebra::Service 0.009 {
 			. $child->as_sparql( %args, level => $level+1 )
 			. "${indent}}\n";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $service	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['SERVICE'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+		my ($child)	= @{ $self->children };
+		
+		my @tokens;
+		push(@tokens, $service);
+		push(@tokens, $self->endpoint->sparql_tokens->elements);
+		push(@tokens, $l);
+		push(@tokens, $child->sparql_tokens->elements);
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Path>
 
 =cut
 
-package Attean::Algebra::Path 0.009 {
+package Attean::Algebra::Path 0.010 {
 	use Moo;
-	use Types::Standard qw(ConsumerOf);
-	sub in_scope_variables {
-		my $self	= shift;
-		my @vars	= map { $_->value } grep { $_->does('Attean::API::Variable') } ($self->subject, $self->object);
-		return Set::Scalar->new(@vars)->members;
-	}
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use Types::Standard qw(ArrayRef ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::Algebra', 'Attean::API::NullaryQueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
 	has 'subject' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
 	has 'path' => (is => 'ro', isa => ConsumerOf['Attean::API::PropertyPath'], required => 1);
 	has 'object' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
+
+	sub in_scope_variables {
+		my $self	= shift;
+		my @vars	= map { $_->value } grep { $_->does('Attean::API::Variable') } ($self->subject, $self->object);
+		return Set::Scalar->new(@vars)->elements;
+	}
+
 	sub tree_attributes { return qw(subject path object) };
 
 	sub algebra_as_string {
@@ -603,15 +847,48 @@ package Attean::Algebra::Path 0.009 {
 			. $self->object->as_sparql
 			. "\n";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+
+		my @tokens;
+		foreach my $t ($self->subject, $self->path, $self->object) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Group>
 
 =cut
 
-package Attean::Algebra::Group 0.009 {
+package Attean::Algebra::Group 0.010 {
+	use utf8;
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
+	use namespace::clean;
+	
+	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
+	has 'groupby' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Expression']]);
+	has 'aggregates' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::AggregateExpression']]);
+	
+	sub BUILD {
+		my $self	= shift;
+		foreach my $a (@{ $self->aggregates }) {
+			my $op	= $a->operator;
+			if ($op eq 'RANK') {
+				if (scalar(@{ $self->aggregates }) > 1) {
+					die "Cannot use both aggregates and RANKing in grouping operator";
+				}
+			}
+		}
+	}
+	
 	sub in_scope_variables {
 		my $self	= shift;
 		my $aggs	= $self->aggregates // [];
@@ -630,14 +907,12 @@ package Attean::Algebra::Group 0.009 {
 			my $v	= $a->variable->as_sparql;
 			my $op	= $a->operator;
 			my $d	= $a->distinct ? "DISTINCT " : '';
-			my ($e)	= map { $_->as_sparql } @{ $a->children };
+			my ($e)	= ((map { $_->as_sparql } @{ $a->children }), '');
 			push(@aggs, "$v ← ${op}($d$e)");
 		}
 		return sprintf('Group { %s } aggregate { %s }', join(', ', map { $_->as_sparql() } @$groups), join(', ', @aggs));
 	}
-	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
-	has 'groupby' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Expression']]);
-	has 'aggregates' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::AggregateExpression']]);
+
 	sub tree_attributes { return qw(groupby aggregates) };
 	sub as_sparql {
 		my $self	= shift;
@@ -675,11 +950,18 @@ package Attean::Algebra::Group 0.009 {
 
 =cut
 
-package Attean::Algebra::NegatedPropertySet 0.009 {
+package Attean::Algebra::NegatedPropertySet 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::PropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	has 'predicates' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::IRI']], required => 1);
+	
 	sub as_string {
 		my $self	= shift;
 		return sprintf("!(%s)", join('|', map { $_->ntriples_string } @{ $self->predicates }));
@@ -690,16 +972,38 @@ package Attean::Algebra::NegatedPropertySet 0.009 {
 		my $self	= shift;
 		return "!(" . join('|', map { $_->as_sparql } @{$self->predicates}) . ")";
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $bang	= AtteanX::SPARQL::Token->fast_constructor( BANG, -1, -1, -1, -1, ['!'] );
+		my $or		= AtteanX::SPARQL::Token->fast_constructor( OR, -1, -1, -1, -1, ['|'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+
+		my @tokens;
+		push(@tokens, $bang, $l);
+		foreach my $t (@{ $self->predicates }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $or);
+		}
+		pop(@tokens); # remove last OR token
+		push(@tokens, $r);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::PredicatePath>
 
 =cut
 
-package Attean::Algebra::PredicatePath 0.009 {
+package Attean::Algebra::PredicatePath 0.010 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::PropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	has 'predicate' => (is => 'ro', isa => ConsumerOf['Attean::API::IRI'], required => 1);
 	sub as_string {
 		my $self	= shift;
@@ -714,117 +1018,272 @@ package Attean::Algebra::PredicatePath 0.009 {
 		my $self	= shift;
 		return $self->predicate->as_sparql;
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		return $self->predicate->sparql_tokens;
+	}
+	
 }
 
 =item * L<Attean::Algebra::InversePath>
 
 =cut
 
-package Attean::Algebra::InversePath 0.009 {
+package Attean::Algebra::InversePath 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::UnaryPropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub prefix_name { return "^" }
 	sub as_sparql {
 		my $self	= shift;
 		my ($path)	= @{ $self->children };
 		return '^' . $self->path->as_sparql;
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $hat		= AtteanX::SPARQL::Token->fast_constructor( HAT, -1, -1, -1, -1, ['^'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		
+		if (scalar(@tokens) > 1) {
+			unshift(@tokens, $hat, $l);
+			push(@tokens, $r);
+		} else {
+			unshift(@tokens, $hat);
+		}
+		
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::SequencePath>
 
 =cut
 
-package Attean::Algebra::SequencePath 0.009 {
+package Attean::Algebra::SequencePath 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use namespace::clean;
+
 	with 'Attean::API::NaryPropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub separator { return "/" }
 	sub as_sparql {
 		my $self	= shift;
 		my @paths	= @{ $self->children };
 		return '(' . join('/', map { $_->as_sparql } @paths) . ')';
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $slash	= AtteanX::SPARQL::Token->fast_constructor( SLASH, -1, -1, -1, -1, ['/'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $slash);
+		}
+		pop(@tokens); # remove last SLASH token
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::AlternativePath>
 
 =cut
 
-package Attean::Algebra::AlternativePath 0.009 {
+package Attean::Algebra::AlternativePath 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use namespace::clean;
+
 	with 'Attean::API::NaryPropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub separator { return "|" }
 	sub as_sparql {
 		my $self	= shift;
 		my @paths	= @{ $self->children };
 		return '(' . join('|', map { $_->as_sparql } @paths) . ')';
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $or		= AtteanX::SPARQL::Token->fast_constructor( OR, -1, -1, -1, -1, ['|'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $or);
+		}
+		pop(@tokens); # remove last OR token
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::ZeroOrMorePath>
 
 =cut
 
-package Attean::Algebra::ZeroOrMorePath 0.009 {
+package Attean::Algebra::ZeroOrMorePath 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::UnaryPropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub postfix_name { return "*" }
 	sub as_sparql {
 		my $self	= shift;
 		my ($path)	= @{ $self->children };
 		return $self->path->as_sparql . '*';
 	}
+	
+	sub sparql_tokens {
+		my $self	= shift;
+		my $star	= AtteanX::SPARQL::Token->fast_constructor( STAR, -1, -1, -1, -1, ['*'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		
+		if (scalar(@tokens) > 1) {
+			unshift(@tokens, $l);
+			push(@tokens, $r);
+		}
+		push(@tokens, $star);
+		
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::OneOrMorePath>
 
 =cut
 
-package Attean::Algebra::OneOrMorePath 0.009 {
+package Attean::Algebra::OneOrMorePath 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::UnaryPropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub postfix_name { return "+" }
 	sub as_sparql {
 		my $self	= shift;
 		my ($path)	= @{ $self->children };
 		return $self->path->as_sparql . '+';
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $plus	= AtteanX::SPARQL::Token->fast_constructor( PLUS, -1, -1, -1, -1, ['+'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		
+		if (scalar(@tokens) > 1) {
+			unshift(@tokens, $l);
+			push(@tokens, $r);
+		}
+		push(@tokens, $plus);
+		
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::ZeroOrOnePath>
 
 =cut
 
-package Attean::Algebra::ZeroOrOnePath 0.009 {
+package Attean::Algebra::ZeroOrOnePath 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
 	with 'Attean::API::UnaryPropertyPath';
+	with 'Attean::API::SPARQLSerializable';
+
 	sub postfix_name { return "?" }
 	sub as_sparql {
 		my $self	= shift;
 		my ($path)	= @{ $self->children };
 		return $self->path->as_sparql . '?';
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $q		= AtteanX::SPARQL::Token->fast_constructor( QUESTION, -1, -1, -1, -1, ['?'] );
+		my $l		= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $r		= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+
+		my @tokens;
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+		}
+		
+		if (scalar(@tokens) > 1) {
+			unshift(@tokens, $l);
+			push(@tokens, $r);
+		}
+		push(@tokens, $q);
+		
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Table>
 
 =cut
 
-package Attean::Algebra::Table 0.009 {
+package Attean::Algebra::Table 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
+
+	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLSerializable';
+
+	has variables => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']]);
+	has rows => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Result']]);
+
 	sub in_scope_variables {
 		my $self	= shift;
 		return map { $_->value } @{ $self->variables };
 	}
-	has variables => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']]);
-	has rows => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Result']]);
-	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
 	sub tree_attributes { return qw(variables rows) };
 	sub algebra_as_string { return 'Table' }
 	sub as_sparql {
@@ -841,16 +1300,52 @@ package Attean::Algebra::Table 0.009 {
 		$sparql		.= "${indent}}\n";
 		return $sparql;
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $values	= AtteanX::SPARQL::Token->fast_constructor( KEYWORD, -1, -1, -1, -1, ['VALUES'] );
+		my $lparen	= AtteanX::SPARQL::Token->fast_constructor( LPAREN, -1, -1, -1, -1, ['('] );
+		my $rparen	= AtteanX::SPARQL::Token->fast_constructor( RPAREN, -1, -1, -1, -1, [')'] );
+		my $lbrace	= AtteanX::SPARQL::Token->fast_constructor( LBRACE, -1, -1, -1, -1, ['{'] );
+		my $rbrace	= AtteanX::SPARQL::Token->fast_constructor( RBRACE, -1, -1, -1, -1, ['}'] );
+
+		my @tokens;
+		push(@tokens, $values, $lparen);
+		foreach my $var (@{ $self->variables }) {
+			push(@tokens, $var->sparql_tokens->elements);
+		}
+		push(@tokens, $values, $rparen);
+		
+		push(@tokens, $values, $lbrace);
+		foreach my $row (@{ $self->rows }) {
+			push(@tokens, $lparen);
+			foreach my $val ($row->values) {
+				# TODO: verify correct serialization of UNDEF
+				push(@tokens, $val->sparql_tokens->elements);
+			}
+			push(@tokens, $rparen);
+		}
+		push(@tokens, $values, $rbrace);
+		
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
 =item * L<Attean::Algebra::Ask>
 
 =cut
 
-package Attean::Algebra::Ask 0.009 {
+package Attean::Algebra::Ask 0.010 {
 	use Moo;
-	sub in_scope_variables { return; }
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use namespace::clean;
+	
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+	
+	sub in_scope_variables { return; }
+
 	sub algebra_as_string { return 'Ask' }
 	sub as_sparql {
 		my $self	= shift;
@@ -869,12 +1364,19 @@ package Attean::Algebra::Ask 0.009 {
 
 =cut
 
-package Attean::Algebra::Construct 0.009 {
+package Attean::Algebra::Construct 0.010 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ArrayRef ConsumerOf);
-	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
-	sub in_scope_variables { return qw(subject predicate object); }
+	use namespace::clean;
+	
 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::SPARQLQuerySerializable';
+
+	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
+
+	sub in_scope_variables { return qw(subject predicate object); }
 	sub tree_attributes { return qw(triples) };
 	sub algebra_as_string { return 'Construct' }
 	sub as_sparql {
