@@ -7,7 +7,7 @@ Attean::Plan - Representation of SPARQL query plan operators
 
 =head1 VERSION
 
-This document describes Attean::Plan version 0.009
+This document describes Attean::Plan version 0.010
 
 =head1 SYNOPSIS
 
@@ -31,9 +31,9 @@ Evaluates a quad pattern against the model.
 
 =cut
 
-package Attean::Plan::Quad 0.009 {
+package Attean::Plan::Quad 0.010 {
 	use Moo;
-	use Scalar::Util qw(reftype);
+	use Scalar::Util qw(blessed reftype);
 	use Types::Standard qw(ConsumerOf ArrayRef);
 	use namespace::clean;
 
@@ -45,6 +45,28 @@ package Attean::Plan::Quad 0.009 {
 	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::NullaryQueryTree';
 	with 'Attean::API::QuadPattern';
 
+	around 'BUILDARGS' => sub {
+		my $orig		= shift;
+		my $class		= shift;
+		my $args		= $orig->( $class, @_ );
+		if (exists $args->{in_scope_variables}) {
+			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
+		}
+		
+		my %vars;
+		foreach my $pos (qw(subject predicate object graph)) {
+			my $term	= $args->{$pos};
+			if (blessed($term) and $term->does('Attean::API::Variable')) {
+				$vars{$term->value}	= $term;
+			}
+		}
+		
+		my @vars	= keys %vars;
+		$args->{in_scope_variables}	= [@vars];
+
+		return $args;
+	};
+
 	sub plan_as_string {
 		my $self	= shift;
 		my @nodes	= $self->values;
@@ -114,7 +136,7 @@ Evaluates a join (natural-, anti-, or left-) using a nested loop.
 
 =cut
 
-package Attean::Plan::NestedLoopJoin 0.009 {
+package Attean::Plan::NestedLoopJoin 0.010 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use namespace::clean;
@@ -201,7 +223,7 @@ Evaluates a join (natural-, anti-, or left-) using a hash join.
 
 =cut
 
-package Attean::Plan::HashJoin 0.009 {
+package Attean::Plan::HashJoin 0.010 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use namespace::clean;
@@ -294,7 +316,7 @@ package Attean::Plan::HashJoin 0.009 {
 
 =cut
 
-package Attean::Plan::Construct 0.009 {
+package Attean::Plan::Construct 0.010 {
 	use Moo;
 	use List::MoreUtils qw(all);
 	use Types::Standard qw(Str ArrayRef ConsumerOf InstanceOf);
@@ -387,28 +409,17 @@ named variable binding.
 
 =cut
 
-package Attean::Plan::EBVFilter 0.009 {
+package Attean::Plan::EBVFilter 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ConsumerOf);
+	use namespace::clean;
+	
 	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
+
 	has 'variable' => (is => 'ro', isa => Str, required => 1);
 
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
-
-		return $class->SUPER::BUILDARGS(%args);
-	}
-	
 	sub plan_as_string {
 		my $self	= shift;
 		return sprintf('EBVFilter { ?%s }', $self->variable);
@@ -457,28 +468,18 @@ ordering.
 
 =cut
 
-package Attean::Plan::Merge 0.009 {
+package Attean::Plan::Merge 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str ArrayRef ConsumerOf);
-	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
-	has 'variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
-	sub plan_as_string { return 'Merge' }
+	use namespace::clean;
 	
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
+	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
 
-		return $class->SUPER::BUILDARGS(%args);
-	}
+	has 'variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
+
+	sub plan_as_string { return 'Merge' }
 	
 	sub impl {
 		my $self	= shift;
@@ -496,26 +497,15 @@ Evaluates a set of sub-plans, returning the union of results.
 
 =cut
 
-package Attean::Plan::Union 0.009 {
+package Attean::Plan::Union 0.010 {
 	use Moo;
 	use Scalar::Util qw(blessed);
-	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::BinaryQueryTree';
-	sub plan_as_string { return 'Union' }
+	use namespace::clean;
 	
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
+	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::BinaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
 
-		return $class->SUPER::BUILDARGS(%args);
-	}
+	sub plan_as_string { return 'Union' }
 	
 	sub impl {
 		my $self	= shift;
@@ -572,7 +562,7 @@ expressions, binding the produced values to new variables.
 
 =cut
 
-package Attean::Plan::Extend 0.009 {
+package Attean::Plan::Extend 0.010 {
 	use Moo;
 	use Encode;
 	use Data::UUID;
@@ -1164,25 +1154,14 @@ hash of already-seen results.
 
 =cut
 
-package Attean::Plan::HashDistinct 0.009 {
+package Attean::Plan::HashDistinct 0.010 {
 	use Moo;
-	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
-	sub plan_as_string { return 'HashDistinct' }
+	use namespace::clean;
 	
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
 
-		return $class->SUPER::BUILDARGS(%args);
-	}
+	sub plan_as_string { return 'HashDistinct' }
 	
 	sub impl {
 		my $self	= shift;
@@ -1203,25 +1182,14 @@ filtering out sequential duplicates.
 
 =cut
 
-package Attean::Plan::Unique 0.009 {
+package Attean::Plan::Unique 0.010 {
 	use Moo;
-	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
-	sub plan_as_string { return 'Unique' }
+	use namespace::clean;
 	
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
 
-		return $class->SUPER::BUILDARGS(%args);
-	}
+	sub plan_as_string { return 'Unique' }
 	
 	sub impl {
 		my $self	= shift;
@@ -1249,28 +1217,17 @@ number of results ("offset") and limiting the total number of returned results
 
 =cut
 
-package Attean::Plan::Slice 0.009 {
+package Attean::Plan::Slice 0.010 {
 	use Moo;
 	use Types::Standard qw(Int);
+	use namespace::clean;
+	
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
+
 	has 'limit' => (is => 'ro', isa => Int, default => -1);
 	has 'offset' => (is => 'ro', isa => Int, default => 0);
 
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
-
-		return $class->SUPER::BUILDARGS(%args);
-	}
-	
 	sub plan_as_string {
 		my $self	= shift;
 		my @str;
@@ -1301,7 +1258,7 @@ of variable bindings in each result.
 
 =cut
 
-package Attean::Plan::Project 0.009 {
+package Attean::Plan::Project 0.010 {
 	use Moo;
 	with 'Attean::API::BindingSubstitutionPlan', 'Attean::API::UnaryQueryTree';
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1376,12 +1333,17 @@ sorting is applied.
 
 =cut
 
-package Attean::Plan::OrderBy 0.009 {
+package Attean::Plan::OrderBy 0.010 {
 	use Moo;
 	use Types::Standard qw(HashRef ArrayRef InstanceOf Bool Str);
+	use namespace::clean;
+	
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
+
 	has 'variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
 	has 'ascending' => (is => 'ro', isa => HashRef[Bool], required => 1);
+
 	sub plan_as_string {
 		my $self	= shift;
 		my @vars	= @{ $self->variables };
@@ -1390,19 +1352,28 @@ package Attean::Plan::OrderBy 0.009 {
 		return sprintf('Order { %s }', join(', ', @strings));
 	}
 	
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
-
-		return $class->SUPER::BUILDARGS(%args);
+	sub sort_rows {
+		my $self		= shift;
+		my $vars		= shift;
+		my $ascending	= shift;
+		my $rows		= shift;
+		my @sorted		= map { $_->[0] } sort {
+			my ($ar, $avalues)	= @$a;
+			my ($br, $bvalues)	= @$b;
+			my $c	= 0;
+			foreach my $i (0 .. $#{ $vars }) {
+				my $ascending	= $ascending->{ $vars->[$i] };
+				my ($av, $bv)	= map { $_->[$i] } ($avalues, $bvalues);
+				$c		= $av ? $av->compare($bv) : 1;
+				$c		*= -1 unless ($ascending);
+				last unless ($c == 0);
+			}
+			$c
+		} map {
+			my $r = $_;
+			[$r, [map { $r->value($_) } @$vars]]
+		} @$rows;
+		return @sorted;
 	}
 	
 	sub impl {
@@ -1414,22 +1385,7 @@ package Attean::Plan::OrderBy 0.009 {
 		return sub {
 			my $iter	= $impl->();
 			my @rows	= $iter->elements;
-			my @sorted	= map { $_->[0] } sort {
-				my ($ar, $avalues)	= @$a;
-				my ($br, $bvalues)	= @$b;
-				my $c	= 0;
-				foreach my $i (0 .. $#{ $vars }) {
-					my $ascending	= $ascending->{ $vars->[$i] };
-					my ($av, $bv)	= map { $_->[$i] } ($avalues, $bvalues);
-					$c		= $av ? $av->compare($bv) : 1;
-					$c		*= -1 unless ($ascending);
-					last unless ($c == 0);
-				}
-				$c
-			} map {
-				my $r = $_;
-				[$r, [map { $r->value($_) } @$vars]]
-			} @rows;
+			my @sorted	= $self->sort_rows($vars, $ascending, \@rows);
 			return Attean::ListIterator->new( values => \@sorted, item_type => $iter->item_type);
 		}
 	}
@@ -1441,19 +1397,23 @@ Evaluates a SPARQL query against a remove endpoint.
 
 =cut
 
-package Attean::Plan::Service 0.009 {
+package Attean::Plan::Service 0.010 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool Str);
+	use namespace::clean;
+
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+
+	has 'endpoint' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
+	has 'silent' => (is => 'ro', isa => Bool, default => 0);
+	has 'sparql' => (is => 'ro', isa => Str, required => 1);
+
 	sub plan_as_string {
 		my $self	= shift;
 		my $sparql	= $self->sparql;
 		$sparql		=~ s/\s+/ /g;
 		return sprintf('Service <%s> %s', $self->endpoint->as_string, $sparql);
 	}
-	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
-	has 'endpoint' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
-	has 'silent' => (is => 'ro', isa => Bool, default => 0);
-	has 'sparql' => (is => 'ro', isa => Str, required => 1);
 	
 	sub tree_attributes { return qw(endpoint) };
 	sub impl {
@@ -1469,15 +1429,28 @@ Returns a constant set of results.
 
 =cut
 
-package Attean::Plan::Table 0.009 {
+package Attean::Plan::Table 0.010 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
+
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+
 	has variables => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']]);
 	has rows => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Result']]);
-	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+
 	sub tree_attributes { return qw(variables rows) };
-	sub plan_as_string { return 'Table' }
+	sub plan_as_string {
+		my $self	= shift;
+		my $level	= shift;
+		my $indent	= '  ' x ($level + 1);
+		my $vars	= join(', ', map { "?$_" } @{ $self->in_scope_variables });
+		my $s		= "Table (" . $vars . ")";
+		foreach my $row (@{ $self->rows }) {
+			$s	.= "\n-${indent} " . $row->as_string;
+		}
+		return $s;
+	}
 	
 	sub BUILDARGS {
 		my $class		= shift;
@@ -1509,7 +1482,7 @@ package Attean::Plan::Table 0.009 {
 
 =cut
 
-package Attean::Plan::ALPPath 0.009 {
+package Attean::Plan::ALPPath 0.010 {
 	use Moo;
 	use Attean::TreeRewriter;
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1650,7 +1623,7 @@ package Attean::Plan::ALPPath 0.009 {
 	}
 }
 
-package Attean::Plan::ZeroOrOnePath 0.009 {
+package Attean::Plan::ZeroOrOnePath 0.010 {
 	use Moo;
 	use Attean::TreeRewriter;
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -1747,31 +1720,20 @@ results were produced by evaluating the sub-plan.
 
 =cut
 
-package Attean::Plan::Exists 0.009 {
+package Attean::Plan::Exists 0.010 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
+	
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	with 'Attean::API::UnionScopeVariablesPlan';
+
 	has variables => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']]);
 	has rows => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Result']]);
-	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+
 	sub tree_attributes { return qw(variables rows) };
 	sub plan_as_string { return 'Exists' }
 	
-	sub BUILDARGS {
-		# TODO: this code is repeated in several plan classes; figure out a way to share it.
-		my $class		= shift;
-		my %args		= @_;
-		my %vars		= map { $_ => 1 } map { @{ $_->in_scope_variables } } @{ $args{ children } };
-		my @vars		= keys %vars;
-		
-		if (exists $args{in_scope_variables}) {
-			Carp::confess "in_scope_variables is computed automatically, and must not be specified in the $class constructor";
-		}
-		$args{in_scope_variables}	= \@vars;
-
-		return $class->SUPER::BUILDARGS(%args);
-	}
-	
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
@@ -1788,7 +1750,7 @@ package Attean::Plan::Exists 0.009 {
 
 =cut
 
-package Attean::Plan::Aggregate 0.009 {
+package Attean::Plan::Aggregate 0.010 {
 	use Moo;
 	use Encode;
 	use Data::UUID;
@@ -1941,6 +1903,12 @@ package Attean::Plan::Aggregate 0.009 {
 			return $group;
 		};
 		
+		my $rank;
+		while (my($var, $agg) = each(%aggs)) {
+			if ($agg->operator eq 'RANK') {
+				$rank	= $var;
+			}
+		}
 		
 		my ($impl)	= map { $_->impl($model) } @{ $self->children };
 		my %row_groups;
@@ -1958,26 +1926,41 @@ package Attean::Plan::Aggregate 0.009 {
 			
 			# SPARQL evaluation of aggregates over an empty input sequence should
 			# result in an empty result <http://answers.semanticweb.com/questions/17410/semantics-of-sparql-aggregates>
+			
+			my @results;
 			unless (scalar(@group_keys)) {
 				push(@group_keys, '');
 				$row_groups{''}			= [];
 				$group_templates{''}	= {};
 			}
-			my @results;
 			foreach my $group (@group_keys) {
 				my %row	= %{ $group_templates{ $group } };
 				my $rows	= $row_groups{$group};
-				foreach my $var (keys %aggs) {
-					my $expr	= $aggs{$var};
-					my $value	= eval { $self->evaluate_aggregate($model, $expr, $rows) };
-					if ($value) {
-						$row{$var}	= $value;
+				if (defined $rank) {
+					my $agg			= $aggs{$rank};
+					my $ascending	= $agg->scalar_vars->{ascending} // {};
+					my $vars	= [map { $_->value->value } @{ $agg->children }];
+					# TODO: support ordering by complex expressions in $vars, not just ValueExpressions with variables
+					my @sorted	= Attean::Plan::OrderBy->sort_rows($vars, $ascending, $rows);
+					my $ord		= 0;
+					foreach my $row (@sorted) {
+						my %b	= %{ $row->bindings };
+						$b{ $rank }	= Attean::Literal->integer($ord++);
+						my $r	= Attean::Result->new( bindings => \%b );
+						push(@results, $r);
 					}
+				} else {
+					foreach my $var (keys %aggs) {
+						my $expr	= $aggs{$var};
+						my $value	= eval { $self->evaluate_aggregate($model, $expr, $rows) };
+						if ($value) {
+							$row{$var}	= $value;
+						}
+					}
+					my $result	= Attean::Result->new( bindings => \%row );
+					push(@results, $result);
 				}
-				my $result	= Attean::Result->new( bindings => \%row );
-				push(@results, $result);
 			}
-			
 			return Attean::ListIterator->new(values => \@results, item_type => 'Attean::API::Result');
 		};
 	}
@@ -1987,7 +1970,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Ask 0.009 {
+# package Attean::Algebra::Ask 0.010 {
 # 	use Moo;
 # 	sub in_scope_variables { return; }
 # 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
@@ -2009,7 +1992,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Construct 0.009 {
+# package Attean::Algebra::Construct 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
@@ -2036,13 +2019,13 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Path 0.009 {
+# package Attean::Algebra::Path 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	sub in_scope_variables {
 # 		my $self	= shift;
 # 		my @vars	= map { $_->value } grep { $_->does('Attean::API::Variable') } ($self->subject, $self->object);
-# 		return Set::Scalar->new(@vars)->members;
+# 		return Set::Scalar->new(@vars)->elements;
 # 	}
 # 	with 'Attean::API::Algebra', 'Attean::API::NullaryQueryTree';
 # 	has 'subject' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable'], required => 1);
@@ -2074,7 +2057,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::NegatedPropertySet 0.009 {
+# package Attean::Algebra::NegatedPropertySet 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -2095,7 +2078,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::PredicatePath 0.009 {
+# package Attean::Algebra::PredicatePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -2119,7 +2102,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::InversePath 0.009 {
+# package Attean::Algebra::InversePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2135,7 +2118,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::SequencePath 0.009 {
+# package Attean::Algebra::SequencePath 0.010 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "/" }
@@ -2150,7 +2133,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::AlternativePath 0.009 {
+# package Attean::Algebra::AlternativePath 0.010 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "|" }
@@ -2165,7 +2148,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrMorePath 0.009 {
+# package Attean::Algebra::ZeroOrMorePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2181,7 +2164,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::OneOrMorePath 0.009 {
+# package Attean::Algebra::OneOrMorePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -2197,7 +2180,7 @@ package Attean::Plan::Aggregate 0.009 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrOnePath 0.009 {
+# package Attean::Algebra::ZeroOrOnePath 0.010 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
