@@ -7,7 +7,7 @@ Attean::QueryPlanner - Query planner
 
 =head1 VERSION
 
-This document describes Attean::QueryPlanner version 0.034
+This document describes Attean::QueryPlanner version 0.010
 
 =head1 SYNOPSIS
 
@@ -17,19 +17,12 @@ This document describes Attean::QueryPlanner version 0.034
   my $default_graphs = [ Attean::IRI->new('http://example.org/') ];
   my $plan = $planner->plan_for_algebra( $algebra, $model, $default_graphs );
   my $iter = $plan->evaluate($model);
-  while (my $result = $iter->next()) {
-    say $result->as_string;
-  }
+  my $iter = $e->evaluate( $model );
 
 =head1 DESCRIPTION
 
-The Attean::QueryPlanner class is a base class implementing common behavior for
-query planners. Subclasses will need to consume or compose the
-L<Attean::API::JoinPlanner> role.
-
-Trivial sub-classes may consume L<Attean::API::NaiveJoinPlanner>, while more
-complex planners may choose to implement complex join planning (e.g.
-L<Attean::IDPQueryPlanner>).
+The Attean::QueryPlanner class implements a query planner using the
+iterative dynamic programming approach.
 
 =head1 ATTRIBUTES
 
@@ -41,13 +34,14 @@ use Attean::Algebra;
 use Attean::Plan;
 use Attean::Expression;
 
-package Attean::QueryPlanner 0.034 {
+package Attean::QueryPlanner 0.010 {
 	use Moo;
 	use Encode qw(encode);
 	use Attean::RDF;
+	use LWP::UserAgent;
 	use Scalar::Util qw(blessed reftype);
 	use List::Util qw(reduce);
-	use List::Util qw(all any);
+	use List::MoreUtils qw(all any);
 	use Types::Standard qw(Int ConsumerOf InstanceOf);
 	use URI::Escape;
 	use Algorithm::Combinatorics qw(subsets);
@@ -55,9 +49,8 @@ package Attean::QueryPlanner 0.034 {
 	use Math::Cartesian::Product;
 	use namespace::clean;
 
-	with 'Attean::API::QueryPlanner', 'MooX::Log::Any';
+	with 'Attean::API::QueryPlanner';
 	has 'counter' => (is => 'rw', isa => Int, default => 0);
-	has 'table_threshold'	=> (is => 'rw', isa => Int, default => 10);
 
 =back
 
@@ -65,12 +58,6 @@ package Attean::QueryPlanner 0.034 {
 
 =over 4
 
-=item C<< new_temporary( $type ) >>
-
-Returns a new unique (in the context of the query planner) ID string that may
-be used for things like fresh (temporary) variables. The C<< $type >> string is
-used in the generated name to aid in identifying different uses for the names.
-
 =cut
 
 	sub new_temporary {
@@ -81,9 +68,11 @@ used in the generated name to aid in identifying different uses for the names.
 		return sprintf('.%s-%d', $type, $c);
 	}
 
-=item C<< plan_for_algebra( $algebra, $model, \@active_graphs, \@default_graphs ) >>
+=item C<< plans_for_algebra( $algebra, $model, \@active_graphs, \@default_graphs ) >>
 
-Returns the first plan returned from C<< plans_for_algebra >>.
+Returns L<Attean::API::Plan> objects representing alternate query plans for
+evaluating the query C<< $algebra >> against the C<< $model >>, using
+the supplied C<< $active_graph >>.
 
 =cut
 
@@ -93,14 +82,6 @@ Returns the first plan returned from C<< plans_for_algebra >>.
 		return shift(@plans);
 	}
 	
-=item C<< plans_for_algebra( $algebra, $model, \@active_graphs, \@default_graphs ) >>
-
-Returns L<Attean::API::Plan> objects representing alternate query plans for
-evaluating the query C<< $algebra >> against the C<< $model >>, using
-the supplied C<< $active_graph >>.
-
-=cut
-
 	sub plans_for_algebra {
 		my $self			= shift;
 		my $algebra			= shift;
@@ -110,11 +91,11 @@ the supplied C<< $active_graph >>.
 		my %args			= @_;
 		
 		if ($model->does('Attean::API::CostPlanner')) {
-			my @plans	= $model->plans_for_algebra($algebra, $self, $active_graphs, $default_graphs, %args);
+			my @plans	= $model->plans_for_algebra($algebra, $model, $active_graphs, $default_graphs, @_);
 			if (@plans) {
 				return @plans; # trust that the model knows better than us what plans are best
 			} else {
- 				$self->log->info("*** Model did not provide plans: $model");
+# 				warn "*** Model did not provide plans: $model";
 			}
 		}
 		
@@ -125,9 +106,7 @@ the supplied C<< $active_graph >>.
 		
 		my @children	= @{ $algebra->children };
 		my ($child)		= $children[0];
-		if ($algebra->isa('Attean::Algebra::Query') or $algebra->isa('Attean::Algebra::Update')) {
-			return $self->plans_for_algebra($algebra->child, $model, $active_graphs, $default_graphs, %args);
-		} elsif ($algebra->isa('Attean::Algebra::BGP')) {
+		if ($algebra->isa('Attean::Algebra::BGP')) {
 			my $triples	= $algebra->triples;
 			my @triples	= @$triples;
 			my %blanks;
@@ -158,10 +137,10 @@ the supplied C<< $active_graph >>.
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Join')) {
 			return $self->group_join_plans($model, $active_graphs, $default_graphs, $interesting, map {
-				[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, %args)]
+				[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)]
 			} @children);
 		} elsif ($algebra->isa('Attean::Algebra::Distinct') or $algebra->isa('Attean::Algebra::Reduced')) {
-			my @plans	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
+			my @plans	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
 			my @dist;
 			foreach my $p (@plans) {
 				if ($p->distinct) {
@@ -207,7 +186,7 @@ the supplied C<< $active_graph >>.
 			my %exprs	= ($var => $expr);
 		
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
 				my $distinct	= $plan->distinct;
 				my $ordered		= $plan->ordered;
 				if ($expr->isa('Attean::ValueExpression') and $expr->value->does('Attean::API::Variable')) {
@@ -217,7 +196,7 @@ the supplied C<< $active_graph >>.
 					my @vars		= ($var);
 					my @inscope		= ($var, @{ $plan->in_scope_variables });
 					my @pvars		= map { Attean::Variable->new($_) } @{ $plan->in_scope_variables };
-					my $extend		= Attean::Plan::Extend->new(children => [$plan], expressions => \%exprs, distinct => 0, ordered => $ordered, active_graphs => $active_graphs);
+					my $extend		= Attean::Plan::Extend->new(children => [$plan], expressions => \%exprs, distinct => 0, ordered => $ordered);
 					my $filtered	= Attean::Plan::EBVFilter->new(children => [$extend], variable => $var, distinct => 0, ordered => $ordered);
 					my $proj		= $self->new_projection($filtered, $distinct, @{ $plan->in_scope_variables });
 					push(@plans, $proj);
@@ -229,7 +208,7 @@ the supplied C<< $active_graph >>.
 			my @cmps	= @{ $algebra->comparators };
 			my ($exprs, $ascending, $svars)	= $self->_order_by($algebra);
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, interesting_order => $algebra->comparators, %args)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, interesting_order => $algebra->comparators, @_)) {
 				my $distinct	= $plan->distinct;
 
 				if (scalar(@cmps) == 1 and $cmps[0]->expression->isa('Attean::ValueExpression') and $cmps[0]->expression->value->does('Attean::API::Variable')) {
@@ -244,7 +223,7 @@ the supplied C<< $active_graph >>.
 				} else {
 					my @vars	= (@{ $plan->in_scope_variables }, keys %$exprs);
 					my @pvars	= map { Attean::Variable->new($_) } @{ $plan->in_scope_variables };
-					my $extend	= Attean::Plan::Extend->new(children => [$plan], expressions => $exprs, distinct => 0, ordered => $plan->ordered, active_graphs => $active_graphs);
+					my $extend	= Attean::Plan::Extend->new(children => [$plan], expressions => $exprs, distinct => 0, ordered => $plan->ordered);
 					my $ordered	= Attean::Plan::OrderBy->new(children => [$extend], variables => $svars, ascending => $ascending, distinct => 0, ordered => \@cmps);
 					my $proj	= $self->new_projection($ordered, $distinct, @{ $plan->in_scope_variables });
 					push(@plans, $proj);
@@ -253,12 +232,12 @@ the supplied C<< $active_graph >>.
 			
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::LeftJoin')) {
-			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, %args)];
-			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, %args)];
+			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, @_)];
+			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, @_)];
 			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 'left', $algebra->expression);
 		} elsif ($algebra->isa('Attean::Algebra::Minus')) {
-			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, %args)];
-			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, %args)];
+			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, @_)];
+			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, @_)];
 			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 'minus');
 		} elsif ($algebra->isa('Attean::Algebra::Project')) {
 			my $vars	= $algebra->variables;
@@ -269,20 +248,12 @@ the supplied C<< $active_graph >>.
 				($vars_key eq join(' ', sort @{ $_->in_scope_variables }))
 					? $_ # no-op if plan is already properly-projected
 					: $self->new_projection($_, $distinct, @vars)
-			} $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
+			} $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Graph')) {
 			my $graph	= $algebra->graph;
 			if ($graph->does('Attean::API::Term')) {
-				if (my $available = $args{available_graphs}) {
-					# the list of available graphs has been restricted, and this
-					# graph is not available so return an empty table plan.
-					unless (any { $_->equals($graph) } @$available) {
-						my $plan	= Attean::Plan::Table->new( variables => [], rows => [], distinct => 0, ordered => [] );
-						return $plan;
-					}
-				}
-				return $self->plans_for_algebra($child, $model, [$graph], $default_graphs, %args);
+				return $self->plans_for_algebra($child, $model, $graph, @_);
 			} else {
 				my $gvar	= $graph->value;
 				my $graphs	= $model->get_graphs;
@@ -291,14 +262,6 @@ the supplied C<< $active_graph >>.
 				$vars{ $gvar }++;
 				my @vars	= keys %vars;
 				
-				my %available;
-				if (my $available = $args{available_graphs}) {
-					foreach my $a (@$available) {
-						$available{ $a->value }++;
-					}
-					$graphs	= $graphs->grep(sub { $available{ $_->value } });
-				}
-				
 				my @branches;
 				my %ignore	= map { $_->value => 1 } @$default_graphs;
 				while (my $graph = $graphs->next) {
@@ -306,8 +269,8 @@ the supplied C<< $active_graph >>.
 					my %exprs	= ($gvar => Attean::ValueExpression->new(value => $graph));
 					# TODO: rewrite $child pattern here to replace any occurrences of the variable $gvar to $graph
 					my @plans	= map {
-						Attean::Plan::Extend->new(children => [$_], expressions => \%exprs, distinct => 0, ordered => $_->ordered, active_graphs => $active_graphs);
-					} $self->plans_for_algebra($child, $model, [$graph], $default_graphs, %args);
+						Attean::Plan::Extend->new(children => [$_], expressions => \%exprs, distinct => 0, ordered => $_->ordered);
+					} $self->plans_for_algebra($child, $model, [$graph], $default_graphs, @_);
 					push(@branches, \@plans);
 				}
 				
@@ -322,37 +285,20 @@ the supplied C<< $active_graph >>.
 			my $rows	= $algebra->rows;
 			my $vars	= $algebra->variables;
 			my @vars	= map { $_->value } @{ $vars };
-			
-			if (scalar(@$rows) < $self->table_threshold) {
-				return Attean::Plan::Table->new( variables => $vars, rows => $rows, distinct => 0, ordered => [] );
-			} else {
-				my $iter	= Attean::ListIterator->new(
-					item_type => 'Attean::API::Result',
-					variables => \@vars,
-					values => $rows
-				);
-				return Attean::Plan::Iterator->new( iterator => $iter, distinct => 0, ordered => [] );
-			}
+			my $plan	= Attean::Plan::Table->new( variables => $vars, rows => $rows, distinct => 0, ordered => [] );
+			return $plan;
 		} elsif ($algebra->isa('Attean::Algebra::Service')) {
 			my $endpoint	= $algebra->endpoint;
 			my $silent		= $algebra->silent;
 			my $sparql		= sprintf('SELECT * WHERE { %s }', $child->as_sparql);
 			my @vars		= $child->in_scope_variables;
-			my $plan		= Attean::Plan::Service->new(
-				request_signer => $self->request_signer,
-				endpoint => $endpoint,
-				silent => $silent,
-				sparql => $sparql,
-				distinct => 0,
-				in_scope_variables => \@vars,
-				ordered => []
-			);
+			my $plan		= Attean::Plan::Service->new( endpoint => $endpoint, silent => $silent, sparql => $sparql, distinct => 0, in_scope_variables => \@vars, ordered => [] );
 			return $plan;
 		} elsif ($algebra->isa('Attean::Algebra::Slice')) {
 			my $limit	= $algebra->limit;
 			my $offset	= $algebra->offset;
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
 				my $vars	= $plan->in_scope_variables;
 				push(@plans, Attean::Plan::Slice->new(children => [$plan], limit => $limit, offset => $offset, distinct => $plan->distinct, ordered => $plan->ordered));
 			}
@@ -360,7 +306,7 @@ the supplied C<< $active_graph >>.
 		} elsif ($algebra->isa('Attean::Algebra::Union')) {
 			# TODO: if both branches are similarly ordered, we can use Attean::Plan::Merge to keep the resulting plan ordered
 			my @vars		= keys %{ { map { map { $_ => 1 } $_->in_scope_variables } @children } };
-			my @plansets	= map { [$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, %args)] } @children;
+			my @plansets	= map { [$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)] } @children;
 
 			my @plans;
 			cartesian {
@@ -374,8 +320,8 @@ the supplied C<< $active_graph >>.
 			my @vars		= $algebra->in_scope_variables;
 
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
-				my $extend		= Attean::Plan::Extend->new(children => [$plan], expressions => \%exprs, distinct => 0, ordered => $plan->ordered, active_graphs => $active_graphs);
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+				my $extend		= Attean::Plan::Extend->new(children => [$plan], expressions => \%exprs, distinct => 0, ordered => $plan->ordered);
 				push(@plans, $extend);
 			}
 			return @plans;
@@ -388,14 +334,14 @@ the supplied C<< $active_graph >>.
 				$exprs{$var}	= $expr;
 			}
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
-				my $extend		= Attean::Plan::Aggregate->new(children => [$plan], aggregates => \%exprs, groups => $groups, distinct => 0, ordered => [], active_graphs => $active_graphs);
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+				my $extend		= Attean::Plan::Aggregate->new(children => [$plan], aggregates => \%exprs, groups => $groups, distinct => 0, ordered => []);
 				push(@plans, $extend);
 			}
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Ask')) {
 			my @plans;
-			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args)) {
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
 				return Attean::Plan::Exists->new(children => [$plan], distinct => 1, ordered => []);
 			}
 			return @plans;
@@ -427,7 +373,7 @@ the supplied C<< $active_graph >>.
 				my @vars	= $algebra->in_scope_variables;
 				
 				my @joins	= $self->group_join_plans($model, $active_graphs, $default_graphs, $interesting, map {
-					[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, %args)]
+					[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)]
 				} @join);
 				
 				my @plans;
@@ -451,7 +397,7 @@ the supplied C<< $active_graph >>.
 				} else {
 					$a		= Attean::Algebra::Path->new( subject => $begin, path => $child, object => $end );
 				}
-				my @cplans	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, %args);
+				my @cplans	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, @_);
 				my @plans;
 				foreach my $cplan (@cplans) {
 					my $plan	= Attean::Plan::ALPPath->new(
@@ -470,7 +416,7 @@ the supplied C<< $active_graph >>.
 				return @plans;
 			} elsif ($path->isa('Attean::Algebra::ZeroOrOnePath')) {
 				my $a		= Attean::Algebra::Path->new( subject => $s, path => $path->children->[0], object => $o );
-				my @children	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, %args);
+				my @children	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, @_);
 				my @plans;
 				foreach my $plan (@children) {
 					push(@plans, Attean::Plan::ZeroOrOnePath->new(
@@ -487,164 +433,13 @@ the supplied C<< $active_graph >>.
 				die "Cannot simplify property path $path: " . $algebra->as_string;
 			}
 		} elsif ($algebra->isa('Attean::Algebra::Construct')) {
-			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
+			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
 			my @plans;
 			foreach my $plan (@children) {
 				push(@plans, Attean::Plan::Construct->new(triples => $algebra->triples, children => [$plan], distinct => 0, ordered => []));
 			}
 			return @plans;
-		} elsif ($algebra->isa('Attean::Algebra::Describe')) {
-			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
-			my @plans;
-			foreach my $plan (@children) {
-				push(@plans, Attean::Plan::Describe->new(terms => $algebra->terms, graph => $active_graphs, children => [$plan], distinct => 0, ordered => []));
-			}
-			return @plans;
-		} elsif ($algebra->isa('Attean::Algebra::Clear')) {
-			my $plan_class	= $algebra->drop ? 'Attean::Plan::Drop' : 'Attean::Plan::Clear';
-			my $target	= $algebra->target;
-			if ($target eq 'GRAPH') {
-				return Attean::Plan::Clear->new(graphs => [$algebra->graph]);
-			} else {
-				my %default	= map { $_->value => 1 } @$active_graphs;
-				my $graphs	= $model->get_graphs;
-				my @graphs;
-				while (my $graph = $graphs->next) {
-					if ($target eq 'ALL') {
-						push(@graphs, $graph);
-					} else {
-						if ($target eq 'DEFAULT' and $default{ $graph->value }) {
-							push(@graphs, $graph);
-						} elsif ($target eq 'NAMED' and not $default{ $graph->value }) {
-							push(@graphs, $graph);
-						}
-					}
-				}
-				return $plan_class->new(graphs => \@graphs);
-			}
-		} elsif ($algebra->isa('Attean::Algebra::Add')) {
-			my $triple	= triplepattern(variable('s'), variable('p'), variable('o'));
-			my $child;
-			my $default_source	= 0;
-			if (my $from = $algebra->source) {
-				($child)	= $self->access_plans( $model, $active_graphs, $triple->as_quad_pattern($from) );
-			} else {
-				$default_source++;
-				my $bgp		= Attean::Algebra::BGP->new( triples => [$triple] );
-				($child)	= $self->plans_for_algebra($bgp, $model, $active_graphs, $default_graphs, %args);
-			}
-			
-			my $dest;
-			my $default_dest	= 0;
-			if (my $g = $algebra->destination) {
-				$dest		= $triple->as_quad_pattern($g);
-			} else {
-				$default_dest++;
-				$dest		= $triple->as_quad_pattern($default_graphs->[0]);
-			}
-
-
-			my @plans;
-			my $run_update	= 1;
-			if ($default_dest and $default_source) {
-				$run_update	= 0;
-			} elsif ($default_dest or $default_source) {
-				#
-			} elsif ($algebra->source->equals($algebra->destination)) {
-				$run_update	= 0;
-			}
-			
-			if ($run_update) {
-				if ($algebra->drop_destination) {
-					my @graphs	= $algebra->has_destination ? $algebra->destination : @$default_graphs;
-					unshift(@plans, Attean::Plan::Clear->new(graphs => [@graphs]));
-				}
-			
-				push(@plans, Attean::Plan::TripleTemplateToModelQuadMethod->new(
-					graph		=> $default_graphs->[0],
-					order		=> ['add_quad'],
-					patterns	=> {'add_quad' => [$dest]},
-					children 	=> [$child],
-				));
-			
-				if ($algebra->drop_source) {
-					my @graphs	= $algebra->has_source ? $algebra->source : @$default_graphs;
-					push(@plans, Attean::Plan::Clear->new(graphs => [@graphs]));
-				}
-			}
-			my $plan	= (scalar(@plans) == 1) ? shift(@plans) : Attean::Plan::Sequence->new( children => \@plans );
-			return $plan;
-		} elsif ($algebra->isa('Attean::Algebra::Modify')) {
-			unless ($child) {
-				# This is an INSERT/DELETE DATA algebra with ground data and no pattern
-				$child	= Attean::Algebra::BGP->new( triples => [] );
-			}
-			
-			my $dataset	= $algebra->dataset;
-			my @default	= @{ $dataset->{default} || [] };
-			my @named	= values %{ $dataset->{named} || {} };
-			
-			my @active_graphs	= @$active_graphs;
-			my @default_graphs	= @$default_graphs;
-			
-			if (scalar(@default) or scalar(@named)) {
-				# change the available named graphs
-				# change the active graph(s)
-				@active_graphs	= @default;
-				@default_graphs	= @default;
-				$args{ available_graphs }	= [@named];
-			} else {
-				# no custom dataset
-			}
-			
-			my @children	= $self->plans_for_algebra($child, $model, \@active_graphs, \@default_graphs, %args);
-			my $i	= $algebra->insert;
-			my $d	= $algebra->delete;
-			my %patterns;
-			my @order;
-			if (scalar(@$d)) {
-				push(@order, 'remove_quad');
-				$patterns{ 'remove_quad' }	= $d;
-			}
-			if (scalar(@$i)) {
-				push(@order, 'add_quad');
-				$patterns{ 'add_quad' }	= $i;
-			}
-			return map {
-				Attean::Plan::TripleTemplateToModelQuadMethod->new(
-					graph		=> $default_graphs->[0],
-					order		=> \@order,
-					patterns	=> \%patterns,
-					children 	=> [$_],
-				)
-			} @children;
-		} elsif ($algebra->isa('Attean::Algebra::Load')) {
-			my $pattern		= triplepattern(variable('subject'), variable('predicate'), variable('object'));
-			my $load		= Attean::Plan::Load->new( url => $algebra->url->value, silent => $algebra->silent );
-			my $graph		= $algebra->has_graph ? $algebra->graph : $default_graphs->[0];
-			my $plan		= Attean::Plan::TripleTemplateToModelQuadMethod->new(
-				graph		=> $graph,
-				order		=> ['add_quad'],
-				patterns	=> {'add_quad' => [$pattern]},
-				children 	=> [$load],
-			);
-			return $plan;
-		} elsif ($algebra->isa('Attean::Algebra::Create')) {
-			return Attean::Plan::Sequence->new( children => [] );
-		} elsif ($algebra->isa('Attean::Algebra::Sequence')) {
-			my @plans;
-			foreach my $child (@{ $algebra->children }) {
-				my ($plan)	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
-				push(@plans, $plan);
-			}
-			return Attean::Plan::Sequence->new( children => \@plans );
-		} elsif ($algebra->isa('Attean::Algebra::Unfold')) {
-			my @plans	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, %args);
-			my @unfold;
-			foreach my $p (@plans) {
-				push(@unfold, Attean::Plan::Unfold->new( children => [$p], expression => $algebra->expression, variables => $algebra->variables, active_graphs => $active_graphs ));
-			}
-			return @unfold;
+		} elsif ($algebra->isa('Attean::Algebra::Group')) {
 		}
 		die "Unimplemented algebra evaluation for: " . $algebra->as_string;
 	}
@@ -682,13 +477,6 @@ the supplied C<< $active_graph >>.
 		}
 	}
 	
-=item C<< simplify_path( $subject, $path, $object ) >>
-
-Return a simplified L<Attean::API::Algebra> object corresponding to the given
-property path.
-
-=cut
-
 	sub simplify_path {
 		my $self	= shift;
 		my $s		= shift;
@@ -713,52 +501,20 @@ property path.
 			my $ra	= $self->_package($self->simplify_path($s, $r, $o));
 			return Attean::Algebra::Union->new( children => [$la, $ra] );
 		} elsif ($path->isa('Attean::Algebra::NegatedPropertySet')) {
-			my @branches;
-			
 			my @preds	= @{ $path->predicates };
-			if (scalar(@preds)) {
-				my $pvar	= Attean::Variable->new(value => $self->new_temporary('nps'));
-				my $pvar_e	= Attean::ValueExpression->new( value => $pvar );
-				my $t		= Attean::TriplePattern->new($s, $pvar, $o);
-				my @vals	= map { Attean::ValueExpression->new( value => $_ ) } @preds;
-				my $expr	= Attean::FunctionExpression->new( children => [$pvar_e, @vals], operator => 'notin' );
-				my $bgp		= Attean::Algebra::BGP->new( triples => [$t] );
-				my $f_fwd	= Attean::Algebra::Filter->new( children => [$bgp], expression => $expr );
-				push(@branches, $f_fwd);
-			}
-
-			my @rev		= @{ $path->reversed };
-			if (scalar(@rev)) {
-				my $pvar	= Attean::Variable->new(value => $self->new_temporary('nps_rev'));
-				my $pvar_e	= Attean::ValueExpression->new( value => $pvar );
-				my $t		= Attean::TriplePattern->new($o, $pvar, $s);
-				my @vals	= map { Attean::ValueExpression->new( value => $_ ) } @rev;
-				my $expr	= Attean::FunctionExpression->new( children => [$pvar_e, @vals], operator => 'notin' );
-				my $bgp		= Attean::Algebra::BGP->new( triples => [$t] );
-				my $f_rev	= Attean::Algebra::Filter->new( children => [$bgp], expression => $expr );
-				push(@branches, $f_rev);
-			}
-
-			if (scalar(@branches) == 1) {
-				return shift(@branches);
-			} else {
-				return Attean::Algebra::Union->new( children => \@branches );
-			}
+			my $pvar	= Attean::Variable->new(value => $self->new_temporary('nps'));
+			my $pvar_e	= Attean::ValueExpression->new( value => $pvar );
+			my $t		= Attean::TriplePattern->new($s, $pvar, $o);
+			my @vals	= map { Attean::ValueExpression->new( value => $_ ) } @preds;
+			my $expr	= Attean::FunctionExpression->new( children => [$pvar_e, @vals], operator => 'notin' );
+			my $bgp		= Attean::Algebra::BGP->new( triples => [$t] );
+			my $f		= Attean::Algebra::Filter->new( children => [$bgp], expression => $expr );
+			return $f;
 		} else {
 			return;
 		}
 	}
 	
-=item C<< new_projection( $plan, $distinct, @variable_names ) >>
-
-Return a new L<< Attean::Plan::Project >> plan over C<< $plan >>, projecting
-the named variables. C<< $disctinct >> should be true if the caller can
-guarantee that the resulting plan will produce distinct results, false otherwise.
-
-This method takes care of computing plan metadata such as the resulting ordering.
-
-=cut
-
 	sub new_projection {
 		my $self		= shift;
 		my $plan		= shift;
@@ -860,14 +616,6 @@ sub-plan participating in the join.
 		return $self->joins_for_plan_alternatives(@_);
 	}
 	
-=item C<< joins_for_plan_alternatives( $model, \@active_graphs, \@default_graphs, $interesting, \@plan_A, \@plan_B, ... ) >>
-
-Returns a list of alternative plans that may all be used to produce results
-matching the join of C<< plan_A >>, C< plan_B >>, etc. Each plan array here
-(e.g. C<< @plan_A >>) should contain equivalent plans.
-
-=cut
-
 	sub joins_for_plan_alternatives {
 		my $self			= shift;
 		my $model			= shift;
@@ -875,17 +623,9 @@ matching the join of C<< plan_A >>, C< plan_B >>, etc. Each plan array here
 		my $default_graphs	= shift;
 		my $interesting		= shift;
 		my @args			= @_; # each $args[$i] here is an array reference containing alternate plans for element $i
-		die "This query planner does not seem to consume a Attean::API::JoinPlanner role (which is necessary for query planning)";
+		die;
 	}
 	
-=item C<< access_plans( $model, $active_graphs, $pattern ) >>
-
-Returns a list of alternative L<Attean::API::Plan> objects that may be used to
-produce results matching the L<Attean::API::TripleOrQuadPattern> $pattern in
-the context of C<< $active_graphs >>.
-
-=cut
-
 	# $pattern is a Attean::API::TripleOrQuadPattern object
 	# Return a Attean::API::Plan object that represents the evaluation of $pattern.
 	# e.g. different plans might represent different ways of producing the matches (table scan, index match, etc.)
@@ -919,17 +659,6 @@ the context of C<< $active_graphs >>.
 		return $plan;
 	}
 	
-=item C<< join_plans( $model, \@active_graphs, \@default_graphs, \@plan_left, \@plan_right, $type [, $expr] ) >>
-
-Returns a list of alternative plans for the join of one plan from C<< @plan_left >>
-and one plan from C<< @plan_right >>. The join C<< $type >> must be one of
-C<< 'inner' >>, C<< 'left' >>, or C<< 'minus' >>, indicating the join algorithm
-to be used. If C<< $type >> is C<< 'left' >>, then the optional C<< $expr >>
-may be used to supply a filter expression that should be used by the SPARQL
-left-join algorithm.
-
-=cut
-
 	# $lhs and $rhs are both Attean::API::Plan objects
 	# Return a Attean::API::Plan object that represents the evaluation of $lhs ⋈ $rhs.
 	# The $left and $minus flags indicate the type of the join to be performed (⟕ and ▷, respectively).
@@ -1047,7 +776,7 @@ at L<https://github.com/kasei/attean/issues>.
 
 =head1 SEE ALSO
 
-
+L<http://www.perlrdf.org/>
 
 =head1 AUTHOR
 
@@ -1055,7 +784,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Gregory Todd Williams.
+Copyright (c) 2014 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
