@@ -7,7 +7,7 @@ AtteanX::Parser::SPARQLLex - SPARQL Lexer
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::SPARQLLex version 0.034
+This document describes AtteanX::Parser::SPARQLLex version 0.010
 
 =head1 SYNOPSIS
 
@@ -17,34 +17,19 @@ This document describes AtteanX::Parser::SPARQLLex version 0.034
 
 ...
 
-=head1 ATTRIBUTES
-
-=over 4
-
-=item C<< canonical_media_type >>
-
-=item C<< media_types >>
-
-=item C<< file_extensions >>
-
-=item C<< handled_type >>
-
-=item C<< extend >>
-
-=back
-
 =head1 METHODS
 
 =over 4
 
 =cut
 
-package AtteanX::Parser::SPARQLLex 0.034 {
+package AtteanX::Parser::SPARQLLex 0.010 {
 	use utf8;
 	use Moo;
 	use Attean;
 	use Encode;
 	use Encode qw(decode);
+	use List::MoreUtils qw(zip);
 	use Types::Standard qw(ArrayRef);
 	use namespace::clean;
 
@@ -103,12 +88,13 @@ the SPARQL query/update read from the L<IO::Handle> object C<< $fh >>.
 	}
 }
 
-package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
+package AtteanX::Parser::SPARQLLex::Iterator 0.010 {
 	use utf8;
 	use Moo;
 	use Attean;
 	use Encode;
 	use Encode qw(decode);
+	use List::MoreUtils qw(zip);
 	use AtteanX::SPARQL::Token;
 	use AtteanX::SPARQL::Constants;
 	use Types::Standard qw(FileHandle Ref Str Int ArrayRef HashRef ConsumerOf InstanceOf);
@@ -147,7 +133,7 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 	my $r_BLANK_NODE_LABEL		= qr/_:${r_PN_LOCAL_BNODE}/o;
 	my $r_ANON					= qr/\[[\t\r\n ]*\]/o;
 	my $r_NIL					= qr/\([\n\r\t ]*\)/o;
-	my $r_KEYWORDS				= qr/(ABS|ADD|ALL|ASC|ASK|AS|AVG|BASE|BIND|BNODE|BOUND|BY|CEIL|CLEAR|COALESCE|CONCAT|CONSTRUCT|CONTAINS|COPY|COUNT|CREATE|DATATYPE|DAY|DEFAULT|DELETE|DELETE WHERE|DESCRIBE|DESC|DISTINCT|DISTINCT|DROP|ENCODE_FOR_URI|EXISTS|FILTER|FLOOR|FROM|GRAPH|GROUP_CONCAT|GROUP|HAVING|HOURS|IF|INSERT|INSERT|DATA|INTO|IN|IRI|ISBLANK|ISIRI|ISLITERAL|ISNUMERIC|ISURI|LANGMATCHES|LANG|LCASE|LIMIT|LOAD|MAX|MD5|MINUS|MINUTES|MIN|MONTH|MOVE|NAMED|NOT|NOW|OFFSET|OPTIONAL|ORDER|PREFIX|RAND|REDUCED|REGEX|REPLACE|ROUND|SAMETERM|SAMPLE|SECONDS|SELECT|SEPARATOR|SERVICE|SHA1|SHA256|SHA384|SHA512|SILENT|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|STRUUID|STR|SUBSTR|SUM|TIMEZONE|TO|TZ|UCASE|UNDEF|UNION|URI|USING|UUID|VALUES|WHERE|WITH|YEAR|TRIPLE|ISTRIPLE|SUBJECT|PREDICATE|OBJECT|HINT|UNFOLD|FOLD)(?!:)\b/io;
+	my $r_KEYWORDS				= qr/(ABS|ADD|ALL|ASC|ASK|AS|AVG|BASE|BIND|BNODE|BOUND|BY|CEIL|CLEAR|COALESCE|CONCAT|CONSTRUCT|CONTAINS|COPY|COUNT|CREATE|DATATYPE|DAY|DEFAULT|DELETE|DELETE WHERE|DESCRIBE|DESC|DISTINCT|DISTINCT|DROP|ENCODE_FOR_URI|EXISTS|FILTER|FLOOR|FROM|GRAPH|GROUP_CONCAT|GROUP|HAVING|HOURS|IF|INSERT|INSERT|DATA|INTO|IN|IRI|ISBLANK|ISIRI|ISLITERAL|ISNUMERIC|ISURI|LANGMATCHES|LANG|LCASE|LIMIT|LOAD|MAX|MD5|MINUS|MINUTES|MIN|MONTH|MOVE|NAMED|NOT|NOW|OFFSET|OPTIONAL|ORDER|PREFIX|RAND|REDUCED|REGEX|REPLACE|ROUND|SAMETERM|SAMPLE|SECONDS|SELECT|SEPARATOR|SERVICE|SHA1|SHA256|SHA384|SHA512|SILENT|STRAFTER|STRBEFORE|STRDT|STRENDS|STRLANG|STRLEN|STRSTARTS|STRUUID|STR|SUBSTR|SUM|TIMEZONE|TO|TZ|UCASE|UNDEF|UNION|URI|USING|UUID|VALUES|WHERE|WITH|YEAR)(?!:)\b/io;
 
 	sub BUILD {
 		my $self	= shift;
@@ -155,15 +141,11 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 		# 	q[#]	=> '_get_comment',
 			q[@]	=> '_get_lang',
 			q[<]	=> '_get_iriref_or_relational',
-			q[{]	=> '_get_brace_or_annotation_or_or',
-			q[}]	=> '_get_brace_or_annotation_or_or',
-			q[|]	=> '_get_brace_or_annotation_or_or',
 			q[_]	=> '_get_bnode',
 			q[']	=> '_get_single_literal',
 			q["]	=> '_get_double_literal',
 			q[:]	=> '_get_pname',
 			q[?]	=> '_get_variable',
-			q[$]	=> '_get_variable',
 			q[!]	=> '_get_bang',
 			q[>]	=> '_get_iriref_or_relational',
 			q([)	=> '_get_lbracket_or_anon',
@@ -181,6 +163,8 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 			','	=> COMMA,
 			'.'	=> DOT,
 			'='	=> EQUALS,
+			'{'	=> LBRACE,
+			'}'	=> RBRACE,
 			']'	=> RBRACKET,
 			')'	=> RPAREN,
 			'-'	=> MINUS,
@@ -223,17 +207,8 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 		unless (length($self->buffer)) {
 			my $line	= $self->file->getline;
 			if (defined($line)) {
-				no warnings 'uninitialized';
-				$line		=~ s{\\(?:(?:u([0-9A-Fa-f]{4}))|(?:U([0-9A-Fa-f]{8})))}{
-					my $h			= $1 . $2;
-					my $codepoint	= hex($h);
-					if ($codepoint >= 0xD800 and $codepoint <= 0xDFFF) {
-						die "Unicode surrogate U+$h is illegal in UTF-8";
-					}
-					chr($codepoint);
-				}ge;
-# 				$line		=~ s/\\u([0-9A-Fa-f]{4})/chr(hex($1))/ge;
-# 				$line		=~ s/\\U([0-9A-Fa-f]{8})/chr(hex($1))/ge;
+				$line		=~ s/\\u([0-9A-Fa-f]{4})/chr(hex($1))/ge;
+				$line		=~ s/\\U([0-9A-Fa-f]{8})/chr(hex($1))/ge;
 				$self->{buffer}	.= $line;
 			}
 		}
@@ -253,15 +228,14 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 		my $self	= shift;
 		my $r		= shift;
 		my $ttype	= shift;
-		my $convert	= shift;
+		my $convert	= shift // sub { shift };
 		my $extend	= $self->extend;
 		push(@$extend, sub {
 			my $l	= shift;
 			
 			if ($l->buffer =~ /^$r\b/) {
 				my $value	= $self->read_length($+[0]);
-				my $c		= $convert ? $convert->($value) : $value;
-				return $l->new_token($ttype, $l->start_line, $l->start_column, $c);
+				return $l->new_token($ttype, $l->start_line, $l->start_column, $convert->($value));
 			}
 		});
 	}
@@ -308,7 +282,7 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 			elsif ($c =~ /[A-Za-z\x{00C0}-\x{00D6}\x{00D8}-\x{00F6}\x{00F8}-\x{02FF}\x{0370}-\x{037D}\x{037F}-\x{1FFF}\x{200C}-\x{200D}\x{2070}-\x{218F}\x{2C00}-\x{2FEF}\x{3001}-\x{D7FF}\x{F900}-\x{FDCF}\x{FDF0}-\x{FFFD}\x{10000}-\x{EFFFF}]/o) {
 				if ($self->buffer =~ /^a(?!:)\s/o) {
 					$self->get_char;
-					return $self->new_token(A, $start_line, $start_column, 'a');
+					return $self->new_token(A, $start_line, $start_column);
 				} elsif ($self->buffer =~ /^(?:true|false)(?!:)\b/o) {
 					my $bool	= $self->read_length($+[0]);
 					return $self->new_token(BOOLEAN, $start_line, $start_column, $bool);
@@ -332,6 +306,14 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 					$self->read_word('^');
 					return $self->new_token(HAT, $start_line, $start_column, '^');
 				}
+			} elsif ($c eq '|') {
+				if ($self->buffer =~ /^\|\|/) {
+					$self->read_word('||');
+					return $self->new_token(OROR, $start_line, $start_column, '||');
+				} else {
+					$self->read_word('|');
+					return $self->new_token(OR, $start_line, $start_column, '|');
+				}
 			} elsif ($c eq '&') {
 				$self->read_word('&&');
 				return $self->new_token(ANDAND, $start_line, $start_column, '&&');
@@ -363,22 +345,12 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 
 	sub _get_variable {
 		my $self	= shift;
-		if (substr($self->buffer, 0, 1) eq '$') {
-			$self->get_char_safe('$');
-			if ($self->buffer =~ /^$r_VARNAME/) {
-				my $name	= $self->read_length($+[0]);
-				return $self->new_token(VAR, $self->start_line, $self->start_column, $name);
-			} else {
-				$self->_throw_error("Invalid variable name");
-			}
+		$self->get_char_safe('?');
+		if ($self->buffer =~ /^$r_VARNAME/) {
+			my $name	= $self->read_length($+[0]);
+			return $self->new_token(VAR, $self->start_line, $self->start_column, $name);
 		} else {
-			$self->get_char_safe('?');
-			if ($self->buffer =~ /^$r_VARNAME/) {
-				my $name	= $self->read_length($+[0]);
-				return $self->new_token(VAR, $self->start_line, $self->start_column, $name);
-			} else {
-				return $self->new_token(QUESTION, $self->start_line, $self->start_column, '?');
-			}
+			return $self->new_token(QUESTION, $self->start_line, $self->start_column, '?');
 		}
 	}
 	
@@ -436,12 +408,6 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 		} elsif (substr($buffer, 0, 2) eq '>=') {
 			$self->read_length(2);
 			return $self->new_token(GE, $self->start_line, $self->start_column, '>=');
-		} elsif (substr($buffer, 0, 2) eq '<<') {
-			$self->read_length(2);
-			return $self->new_token(LTLT, $self->start_line, $self->start_column, '<<');
-		} elsif (substr($buffer, 0, 2) eq '>>') {
-			$self->read_length(2);
-			return $self->new_token(GTGT, $self->start_line, $self->start_column, '>>');
 		} elsif (substr($buffer, 0, 1) eq '>') {
 			$self->get_char;
 			return $self->new_token(GT, $self->start_line, $self->start_column, '>');
@@ -497,29 +463,6 @@ package AtteanX::Parser::SPARQLLex::Iterator 0.034 {
 		}
 	}
 	
-	sub _get_brace_or_annotation_or_or {
-		my $self	= shift;
-		if (substr($self->buffer, 0, 2) eq '{|') {
-			$self->read_length(2);
-			return $self->new_token(LANNOT, $self->start_line, $self->start_column, '{|');
-		} elsif (substr($self->buffer, 0, 2) eq '|}') {
-			$self->read_length(2);
-			return $self->new_token(RANNOT, $self->start_line, $self->start_column, '|}');
-		} elsif (substr($self->buffer, 0, 2) eq '||') {
-			$self->read_length(2);
-			return $self->new_token(OROR, $self->start_line, $self->start_column, '||');
-		} elsif (substr($self->buffer, 0, 1) eq '{') {
-			$self->get_char_safe('{');
-			return $self->new_token(LBRACE, $self->start_line, $self->start_column, '{');
-		} elsif (substr($self->buffer, 0, 1) eq '}') {
-			$self->get_char_safe('}');
-			return $self->new_token(RBRACE, $self->start_line, $self->start_column, '}');
-		} else {
-			$self->get_char_safe('|');
-			return $self->new_token(OR, $self->start_line, $self->start_column, '|');
-		}
-	}
-
 	sub _get_lbracket_or_anon {
 		my $self	= shift;
 		if ($self->buffer =~ /^$r_ANON/) {
@@ -712,7 +655,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Gregory Todd Williams. This
+Copyright (c) 2006-2012 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
