@@ -7,7 +7,7 @@ AtteanX::SPARQL::Constants - Constant definitions for use in parsing and seriali
 
 =head1 VERSION
 
-This document describes AtteanX::SPARQL::Constants version 0.034
+This document describes AtteanX::SPARQL::Constants version 0.010
 
 =head1 SYNOPSIS
 
@@ -19,23 +19,40 @@ This document describes AtteanX::SPARQL::Constants version 0.034
 
 =cut
 
-package AtteanX::SPARQL::Constants 0.034 {
+package AtteanX::SPARQL::Constants 0.010 {
 	use v5.14;
 	use warnings;
-	use AtteanX::Parser::Turtle::Constants;
-	
+
+	our $VERSION;
 	our @EXPORT;
-	our @LOCAL_TYPES;
 	BEGIN {
-		@LOCAL_TYPES	= qw(
+		$VERSION				= 0.010;
+		@EXPORT = qw(
+			A
 			ANDAND
 			ANON
 			BANG
+			BASE
+			BNODE
+			BOOLEAN
+			COMMA
+			COMMENT
+			DECIMAL
+			DOT
+			DOUBLE
+			EQUALS
 			GE
 			GT
 			HAT
+			HATHAT
+			INTEGER
+			IRI
 			KEYWORD
+			LANG
+			LBRACE
+			LBRACKET
 			LE
+			LPAREN
 			LT
 			MINUS
 			NIL
@@ -43,17 +60,23 @@ package AtteanX::SPARQL::Constants 0.034 {
 			OR
 			OROR
 			PLUS
+			PREFIX
+			PREFIXNAME
 			QUESTION
+			RBRACE
+			RBRACKET
+			RPAREN
+			SEMICOLON
 			SLASH
 			STAR
+			STRING1D
+			STRING1S
+			STRING3D
+			STRING3S
 			VAR
+			WS
 			decrypt_constant
-		);
-# 			GTGT
-# 			LTLT
-# 			LANNOT
-# 			RANNOT
-		@EXPORT = (@AtteanX::Parser::Turtle::Constants::EXPORT, @LOCAL_TYPES);
+		)
 	};
 	use base 'Exporter';
 
@@ -61,8 +84,8 @@ package AtteanX::SPARQL::Constants 0.034 {
 		my %mapping;
 		my %reverse;
 		BEGIN {
-			my $cx	= scalar(@AtteanX::Parser::Turtle::Constants::EXPORT) - 1;
-			foreach my $name (grep { $_ ne 'decrypt_constant' } @LOCAL_TYPES) {
+			my $cx	= 0;
+			foreach my $name (grep { $_ ne 'decrypt_constant' } @EXPORT) {
 				my $value	= ++$cx;
 				$reverse{ $value }	= $name;
 				$mapping{ $name }	= $value;
@@ -76,15 +99,7 @@ Returns the token name for the given token type.
 
 =cut
 
-		no warnings 'redefine';
-		sub decrypt_constant {
-			my $num	= +shift;
-			if (exists $reverse{$num}) {
-				return $reverse{$num};
-			} else {
-				return AtteanX::Parser::Turtle::Constants::decrypt_constant($num);
-			}
-		}
+		sub decrypt_constant { my $num	= +shift; $reverse{$num} }
 	}
 }
 
@@ -103,11 +118,9 @@ at L<https://github.com/kasei/perlrdf/issues>.
 
 Toby Inkster C<< <tobyink@cpan.org> >>
 
-Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
-
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Toby Inkster. This
+Copyright (c) 2012 Toby Inkster. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
