@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQLHTML - SPARQL Results HTML Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQLHTML version 0.034
+This document describes AtteanX::Serializer::SPARQLHTML version 0.010
 
 =head1 SYNOPSIS
 
@@ -16,16 +16,6 @@ This document describes AtteanX::Serializer::SPARQLHTML version 0.034
 
 ...
 
-=head1 ATTRIBUTES
-
-=over 4
-
-=item C<< canonical_media_type >>
-
-=item C<< file_extensions >>
-
-=back
-
 =head1 METHODS
 
 =over 4
@@ -35,13 +25,13 @@ This document describes AtteanX::Serializer::SPARQLHTML version 0.034
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQLHTML 0.034 {
+package AtteanX::Serializer::SPARQLHTML 0.010 {
 	use Moo;
 	use Types::Standard qw(Str Bool ArrayRef);
 	use Encode qw(encode);
 	use Scalar::Util qw(blessed);
 	use Attean::ListIterator;
-	use List::Util qw(any);
+	use List::MoreUtils qw(any);
 	use namespace::clean;
 
 	has 'full_document' => (is => 'rw', isa => Bool, default => 1);
@@ -56,14 +46,6 @@ Returns a list of media types that identify the format produced by this serializ
 	sub media_types {
 		return [qw(text/html)];
 	}
-
-=item C<< file_extensions >>
-
-Returns a list of file extensions associated with the serialized format.
-
-=cut
-
-	sub file_extensions { return [qw(html)] };
 	
 =item C<< serialize_iter_to_io( $fh, $iterator ) >>
 
@@ -117,7 +99,7 @@ END
 		return;
 	}
 	
-=item C<< serialize_iter_to_bytes( $iterator ) >>
+=item C<< serialize_iter_to_bytes( $fh, $iterator ) >>
 
 Serializes the L<Attean::API::Binding> objects from C<< $iterator >>
 and returns the serialization as a UTF-8 encoded byte string.
@@ -129,17 +111,11 @@ and returns the serialization as a UTF-8 encoded byte string.
 		my $iter	= shift;
 		my $data	= '';
 		open(my $fh, '>:utf8', \$data);
-		$self->serialize_iter_to_io($fh, $iter);
+		return $self->serialize_iter_to_io($fh, $iter);
 		close($fh);
 		return $data;
 	}
 
-=item C<< node_as_html($node) >>
-
-Serializes the L<Attean::API::Term> object as HTML.
-
-=cut
-
 	sub node_as_html {
 		my $self	= shift;
 		my $node	= shift;
@@ -150,26 +126,11 @@ Serializes the L<Attean::API::Term> object as HTML.
 				s/&/&amp;/g;
 				s/</&lt;/g;
 			}
-			my $html = qq[<a href="${uri}">$uri</a>];
-
-			if (my $map = $self->namespaces) {
-				my $abr = $map->abbreviate($uri);
-
-				if ($abr) {
-					return qq[<a href="${uri}">$abr</a>];
-				} else {
-					return $html;
-				}
-
-			} else {
-			
-				return $html;
-			}
-
-
+			my $html	= $uri;
 # 			if ($link) {
 # 				$html	= qq[<a href="${uri}">$html</a>];
 # 			}
+			return $html;
 		} elsif ($node->does('Attean::API::Literal')) {
 			my $html	= $node->value;
 			for ($html) {
@@ -187,7 +148,6 @@ Serializes the L<Attean::API::Term> object as HTML.
 		}
 	}
 	with 'Attean::API::ResultSerializer';
-	with 'Attean::API::AbbreviatingSerializer';
 }
 
 1;
@@ -207,7 +167,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2022 Gregory Todd Williams. This
+Copyright (c) 2006-2012 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
