@@ -4,7 +4,7 @@ AtteanX::Serializer::SPARQLXML - SPARQL Results XML Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::SPARQLXML version 0.007
+This document describes AtteanX::Serializer::SPARQLXML version 0.010
 
 =head1 SYNOPSIS
 
@@ -25,7 +25,7 @@ This document describes AtteanX::Serializer::SPARQLXML version 0.007
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::SPARQLXML 0.007 {
+package AtteanX::Serializer::SPARQLXML 0.010 {
 	use Moo;
 	use Types::Standard qw(Str ArrayRef);
 	use Encode qw(encode);
@@ -129,8 +129,6 @@ and returns the serialization as a UTF-8 encoded byte string.
 		open(my $fh, '>', \$data);
 		$self->serialize_iter_to_io($fh, $iter);
 		close($fh);
-		use Devel::Peek;
-		Dump($data);
 		return $data;
 	}
 
